@@ -1,9 +1,9 @@
 /*
- * "$Id: cgi-private.h 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: cgi-private.h 9042 2010-03-24 00:45:34Z mike $"
  *
- *   Private CGI definitions for the Common UNIX Printing System (CUPS).
+ *   Private CGI definitions for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -18,18 +18,18 @@
  */
 
 #include "cgi.h"
-#include <cups/debug.h>
-#include <cups/i18n.h>
-#include <cups/string.h>
+#include <cups/debug-private.h>
+#include <cups/language-private.h>
+#include <cups/string-private.h>
 
 
 /*
  * Limits...
  */
 
-#define CUPS_PAGE_MAX	10		/* Maximum items per page */
+#define CUPS_PAGE_MAX	100		/* Maximum items per page */
 
 
 /*
- * End of "$Id: cgi-private.h 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: cgi-private.h 9042 2010-03-24 00:45:34Z mike $".
  */
