@@ -1,10 +1,9 @@
 /*
- * "$Id: rastertoepson.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: rastertoepson.c 9384 2010-11-22 07:06:39Z mike $"
  *
- *   EPSON ESC/P and ESC/P2 filter for the Common UNIX Printing System
- *   (CUPS).
+ *   EPSON ESC/P and ESC/P2 filter for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -32,10 +31,9 @@
 
 #include <cups/cups.h>
 #include <cups/ppd.h>
-#include <cups/string.h>
-#include <cups/i18n.h>
-#include "raster.h"
-#include <stdlib.h>
+#include <cups/string-private.h>
+#include <cups/language-private.h>
+#include <cups/raster.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -104,7 +102,7 @@ void	OutputRows(const cups_page_header2_t *header, int row);
 void
 Setup(void)
 {
-  const char	*device_uri;	/* The device for the printer... */
+  const char	*device_uri;		/* The device for the printer... */
 
 
  /*
@@ -123,11 +121,12 @@ Setup(void)
  */
 
 void
-StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
-          const cups_page_header2_t *header)	/* I - Page header */
+StartPage(
+    const ppd_file_t         *ppd,	/* I - PPD file */
+    const cups_page_header2_t *header)	/* I - Page header */
 {
-  int	n, t;					/* Numbers */
-  int	plane;					/* Looping var */
+  int	n, t;				/* Numbers */
+  int	plane;				/* Looping var */
 
 
  /*
@@ -143,8 +142,6 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
   * See which type of printer we are using...
   */
 
-  EjectPage = header->Margins[0] || header->Margins[1];
-    
   switch (Model)
   {
     case EPSON_9PIN :
@@ -164,8 +161,8 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 
 	printf("\033l%c\033Q%c", 0,	/* Side margins */
                       (int)(10.0 * header->PageSize[0] / 72.0 + 0.5));
-	printf("\033C%c%c", 0,		/* Page length */
-                      (int)(header->PageSize[1] / 72.0 + 0.5));
+	printf("\033\062\033C%c",	/* Page length in 1/6th inches */
+		      (int)(header->PageSize[1] / 12.0 + 0.5));
 	printf("\033N%c", 0);		/* Bottom margin */
         printf("\033O");		/* No perforation skip */
 
@@ -279,7 +276,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 
   if ((Planes[0] = malloc(header->cupsBytesPerLine)) == NULL)
   {
-    fputs("ERROR: Unable to allocate memory!\n", stderr);
+    fputs("ERROR: Unable to allocate memory\n", stderr);
     exit(1);
   }
 
@@ -290,7 +287,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
   {
     if ((CompBuffer = calloc(2, header->cupsWidth)) == NULL)
     {
-      fputs("ERROR: Unable to allocate memory!\n", stderr);
+      fputs("ERROR: Unable to allocate memory\n", stderr);
       exit(1);
     }
   }
@@ -302,7 +299,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
     if ((LineBuffers[0] = calloc(DotBytes,
                                  header->cupsWidth * (Shingling + 1))) == NULL)
     {
-      fputs("ERROR: Unable to allocate memory!\n", stderr);
+      fputs("ERROR: Unable to allocate memory\n", stderr);
       exit(1);
     }
 
@@ -320,7 +317,8 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
  */
 
 void
-EndPage(const cups_page_header2_t *header)	/* I - Page header */
+EndPage(
+    const cups_page_header2_t *header)	/* I - Page header */
 {
   if (DotBytes && header)
   {
@@ -349,8 +347,7 @@ EndPage(const cups_page_header2_t *header)	/* I - Page header */
   * Eject the current page...
   */
 
-  if (EjectPage)
-    putchar(12);		/* Form feed */
+  putchar(12);				/* Form feed */
   fflush(stdout);
 
  /*
@@ -608,7 +605,8 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
  */
 
 void
-OutputLine(const cups_page_header2_t *header)	/* I - Page header */
+OutputLine(
+    const cups_page_header2_t *header)	/* I - Page header */
 {
   if (header->cupsRowCount)
   {
@@ -776,14 +774,15 @@ OutputLine(const cups_page_header2_t *header)	/* I - Page header */
  */
 
 void
-OutputRows(const cups_page_header2_t *header,	/* I - Page image header */
-           int                      row)	/* I - Row number (0 or 1) */
+OutputRows(
+    const cups_page_header2_t *header,	/* I - Page image header */
+    int                      row)	/* I - Row number (0 or 1) */
 {
-  unsigned	i, n;				/* Looping vars */
-  int		dot_count,			/* Number of bytes to print */
-                dot_min;			/* Minimum number of bytes */
-  unsigned char *dot_ptr,			/* Pointer to print data */
-		*ptr;				/* Current data */
+  unsigned	i, n;			/* Looping vars */
+  int		dot_count,		/* Number of bytes to print */
+                dot_min;		/* Minimum number of bytes */
+  unsigned char *dot_ptr,		/* Pointer to print data */
+		*ptr;			/* Current data */
 
 
   dot_min = DotBytes * DotColumns;
@@ -982,8 +981,9 @@ main(int  argc,				/* I - Number of command-line arguments */
     * and return.
     */
 
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            argv[0]);
+    _cupsLangPrintFilter(stderr, "ERROR",
+                         _("%s job-id user title copies options [file]"),
+                         "rastertoepson");
     return (1);
   }
 
@@ -995,7 +995,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   {
     if ((fd = open(argv[6], O_RDONLY)) == -1)
     {
-      perror("ERROR: Unable to open raster file - ");
+      _cupsLangPrintError("ERROR", _("Unable to open raster file"));
       sleep(1);
       return (1);
     }
@@ -1029,8 +1029,22 @@ main(int  argc,				/* I - Number of command-line arguments */
   */
 
   ppd = ppdOpenFile(getenv("PPD"));
-  if (ppd)
-    Model = ppd->model_number;
+  if (!ppd)
+  {
+    ppd_status_t	status;		/* PPD error */
+    int			linenum;	/* Line number */
+
+    _cupsLangPrintFilter(stderr, "ERROR",
+                         _("The PPD file could not be opened."));
+
+    status = ppdLastError(&linenum);
+
+    fprintf(stderr, "DEBUG: %s on line %d.\n", ppdErrorString(status), linenum);
+
+    return (1);
+  }
+
+  Model = ppd->model_number;
 
   Setup();
 
@@ -1052,6 +1066,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     page ++;
 
     fprintf(stderr, "PAGE: %d %d\n", page, header.NumCopies);
+    _cupsLangPrintFilter(stderr, "INFO", _("Starting page %d."), page);
 
    /*
     * Start the page...
@@ -1073,8 +1088,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	break;
 
       if ((y & 127) == 0)
-        fprintf(stderr, _("INFO: Printing page %d, %d%% complete...\n"), page,
-	        100 * y / header.cupsHeight);
+      {
+        _cupsLangPrintFilter(stderr, "INFO",
+	                     _("Printing page %d, %d%% complete."),
+			     page, 100 * y / header.cupsHeight);
+        fprintf(stderr, "ATTR: job-media-progress=%d\n",
+		100 * y / header.cupsHeight);
+      }
 
      /*
       * Read a line of graphics...
@@ -1094,6 +1114,8 @@ main(int  argc,				/* I - Number of command-line arguments */
     * Eject the page...
     */
 
+    _cupsLangPrintFilter(stderr, "INFO", _("Finished page %d."), page);
+
     EndPage(&header);
 
     if (Canceled)
@@ -1121,14 +1143,18 @@ main(int  argc,				/* I - Number of command-line arguments */
   */
 
   if (page == 0)
-    fputs(_("ERROR: No pages found!\n"), stderr);
+  {
+    _cupsLangPrintFilter(stderr, "ERROR", _("No pages were found."));
+    return (1);
+  }
   else
-    fputs(_("INFO: Ready to print.\n"), stderr);
-
-  return (page == 0);
+  {
+    _cupsLangPrintFilter(stderr, "INFO", _("Ready to print."));
+    return (0);
+  }
 }
 
 
 /*
- * End of "$Id: rastertoepson.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: rastertoepson.c 9384 2010-11-22 07:06:39Z mike $".
  */
