@@ -1,9 +1,9 @@
 /*
- * "$Id: textcommon.c 8039 2008-10-09 20:12:03Z mike $"
+ * "$Id: textcommon.c 9793 2011-05-20 03:49:49Z mike $"
  *
- *   Common text filter routines for the Common UNIX Printing System (CUPS).
+ *   Common text filter routines for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -26,7 +26,7 @@
  */
 
 #include "textcommon.h"
-#include <cups/i18n.h>
+#include <cups/language-private.h>
 
 
 /*
@@ -45,7 +45,6 @@ lchar_t	**Page = NULL;		/* Page characters */
 int	NumPages = 0;		/* Number of pages in document */
 float	CharsPerInch = 10;	/* Number of character columns per inch */
 float	LinesPerInch = 6;	/* Number of lines per inch */
-int	UTF8 = 0;		/* Use UTF-8 encoding? */
 int	NumKeywords = 0;	/* Number of known keywords */
 char	**Keywords = NULL;	/* List of known keywords */
 
@@ -515,8 +514,9 @@ TextMain(const char *name,	/* I - Name of filter */
 
   if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            name);
+    _cupsLangPrintf(stderr,
+                    _("Usage: %s job-id user title copies options [file]"),
+                    name);
     return (1);
   }
 
@@ -548,8 +548,8 @@ TextMain(const char *name,	/* I - Name of filter */
   num_options = cupsParseOptions(argv[5], 0, &options);
 
   if ((val = cupsGetOption("prettyprint", num_options, options)) != NULL &&
-      strcasecmp(val, "no") && strcasecmp(val, "off") &&
-      strcasecmp(val, "false"))
+      _cups_strcasecmp(val, "no") && _cups_strcasecmp(val, "off") &&
+      _cups_strcasecmp(val, "false"))
   {
     PageLeft     = 72.0f;
     PageRight    = PageWidth - 36.0f;
@@ -564,25 +564,25 @@ TextMain(const char *name,	/* I - Name of filter */
       NumKeywords = 0;
       Keywords    = NULL;
     }
-    else if (strcasecmp(val, "application/x-cshell") == 0)
+    else if (_cups_strcasecmp(val, "application/x-cshell") == 0)
     {
       PrettyPrint = PRETTY_SHELL;
       NumKeywords = sizeof(csh_keywords) / sizeof(csh_keywords[0]);
       Keywords    = csh_keywords;
     }
-    else if (strcasecmp(val, "application/x-csource") == 0)
+    else if (_cups_strcasecmp(val, "application/x-csource") == 0)
     {
       PrettyPrint = PRETTY_CODE;
       NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
       Keywords    = code_keywords;
     }
-    else if (strcasecmp(val, "application/x-perl") == 0)
+    else if (_cups_strcasecmp(val, "application/x-perl") == 0)
     {
       PrettyPrint = PRETTY_PERL;
       NumKeywords = sizeof(perl_keywords) / sizeof(perl_keywords[0]);
       Keywords    = perl_keywords;
     }
-    else if (strcasecmp(val, "application/x-shell") == 0)
+    else if (_cups_strcasecmp(val, "application/x-shell") == 0)
     {
       PrettyPrint = PRETTY_SHELL;
       NumKeywords = sizeof(sh_keywords) / sizeof(sh_keywords[0]);
@@ -601,8 +601,8 @@ TextMain(const char *name,	/* I - Name of filter */
   if ((val = cupsGetOption("wrap", num_options, options)) == NULL)
     WrapLines = 1;
   else
-    WrapLines = !strcasecmp(val, "true") || !strcasecmp(val, "on") ||
-                !strcasecmp(val, "yes");
+    WrapLines = !_cups_strcasecmp(val, "true") || !_cups_strcasecmp(val, "on") ||
+                !_cups_strcasecmp(val, "yes");
 
   if ((val = cupsGetOption("columns", num_options, options)) != NULL)
   {
@@ -610,7 +610,8 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if (PageColumns < 1)
     {
-      _cupsLangPrintf(stderr, _("ERROR: Bad columns value %d!\n"), PageColumns);
+      _cupsLangPrintFilter(stderr, "ERROR", _("Bad columns value %d."),
+                           PageColumns);
       return (1);
     }
   }
@@ -621,7 +622,8 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if (CharsPerInch <= 0.0)
     {
-      _cupsLangPrintf(stderr, _("ERROR: Bad cpi value %f!\n"), CharsPerInch);
+      _cupsLangPrintFilter(stderr, "ERROR", _("Bad cpi value %f."),
+                           CharsPerInch);
       return (1);
     }
   }
@@ -632,7 +634,8 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if (LinesPerInch <= 0.0)
     {
-      _cupsLangPrintf(stderr, _("ERROR: Bad lpi value %f!\n"), LinesPerInch);
+      _cupsLangPrintFilter(stderr, "ERROR", _("Bad lpi value %f."),
+                           LinesPerInch);
       return (1);
     }
   }
@@ -1089,7 +1092,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	  }
 
           column ++;
-          break;          
+          break;
     }
 
    /*
@@ -1166,7 +1169,7 @@ getutf8(FILE *fp)	/* I - File to read from */
   if ((ch = getc(fp)) == EOF)
     return (EOF);
 
-  if (ch < 0xc0 || !UTF8)	/* One byte character? */
+  if (ch < 0xc0)			/* One byte character? */
     return (ch);
   else if ((ch & 0xe0) == 0xc0)
   {
@@ -1207,5 +1210,5 @@ getutf8(FILE *fp)	/* I - File to read from */
 
 
 /*
- * End of "$Id: textcommon.c 8039 2008-10-09 20:12:03Z mike $".
+ * End of "$Id: textcommon.c 9793 2011-05-20 03:49:49Z mike $".
  */
