@@ -1,9 +1,9 @@
 /*
- * "$Id: cupsfilter.c 7864 2008-08-26 03:43:28Z mike $"
+ * "$Id: cupsfilter.c 9862 2011-08-03 02:44:09Z mike $"
  *
- *   CUPS filtering program for the Common UNIX Printing System (CUPS).
+ *   Filtering program for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -14,33 +14,39 @@
  *
  * Contents:
  *
- *   main()            - Main entry for the test program.
- *   compare_pids()    - Compare two filter PIDs...
- *   escape_options()  - Convert an options array to a string.
- *   exec_filter()     - Execute a single filter.
- *   exec_filters()    - Execute filters for the given file and options.
- *   open_pipe()       - Create a pipe which is closed on exec.
- *   read_cupsd_conf() - Read the cupsd.conf file to get the filter settings.
- *   set_string()      - Copy and set a string.
- *   usage()           - Show program usage...
+ *   main()                - Main entry for the test program.
+ *   add_printer_filter()  - Add a single filters from a PPD file.
+ *   add_printer_filters() - Add filters from a PPD file.
+ *   check_cb()            - Callback function for _cupsFileCheck.
+ *   compare_pids()        - Compare two filter PIDs...
+ *   escape_options()      - Convert an options array to a string.
+ *   exec_filter()         - Execute a single filter.
+ *   exec_filters()        - Execute filters for the given file and options.
+ *   get_job_file()        - Get the specified job file.
+ *   open_pipe()           - Create a pipe which is closed on exec.
+ *   read_cupsd_conf()     - Read the cupsd.conf file to get the filter
+ *                           settings.
+ *   set_string()          - Copy and set a string.
+ *   sighandler()          - Signal catcher for when we print from stdin...
+ *   usage()               - Show program usage...
  */
 
 /*
  * Include necessary headers...
  */
 
-#include <cups/cups.h>
-#include <cups/i18n.h>
-#include <cups/string.h>
-#include <errno.h>
+#include <cups/cups-private.h>
+#include <cups/file-private.h>
+#include <cups/ppd-private.h>
 #include "mime.h"
-#include <stdlib.h>
+#include <limits.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <sys/wait.h>
 #if defined(__APPLE__)
 #  include <libgen.h>
-#endif /* __APPLE__ */ 
+#endif /* __APPLE__ */
 
 
 /*
@@ -63,26 +69,40 @@ static char		*ServerBin = NULL;
 static char		*ServerRoot = NULL;
 					/* CUPS_SERVERROOT environment variable */
 static char		*RIPCache = NULL;
-					/* RIP_CACHE environment variable */
+					/* RIP_MAX_CACHE environment variable */
+static char		TempFile[1024] = "";
+					/* Temporary file */
 
 
 /*
  * Local functions...
  */
 
-static int	compare_pids(mime_filter_t *a, mime_filter_t *b);
-static char	*escape_options(int num_options, cups_option_t *options);
-static int	exec_filter(const char *filter, char **argv, char **envp,
-		            int infd, int outfd);
-static int	exec_filters(cups_array_t *filters, const char *infile,
-		             const char *outfile, const char *ppdfile,
-			     const char *printer, const char *user,
-			     const char *title, int num_options,
-			     cups_option_t *options);
-static int	open_pipe(int *fds);
-static int	read_cupsd_conf(const char *filename);
-static void	set_string(char **s, const char *val);
-static void	usage(const char *command, const char *opt);
+static void		add_printer_filter(const char *command, mime_t *mime,
+			                   mime_type_t *printer_type,
+		                           const char  *filter);
+static mime_type_t	*add_printer_filters(const char *command,
+					     mime_t *mime, const char *printer,
+			                     const char *ppdfile,
+					     mime_type_t **prefilter_type);
+static void		check_cb(void *context, _cups_fc_result_t result,
+				 const char *message);
+static int		compare_pids(mime_filter_t *a, mime_filter_t *b);
+static char		*escape_options(int num_options, cups_option_t *options);
+static int		exec_filter(const char *filter, char **argv,
+			            char **envp, int infd, int outfd);
+static int		exec_filters(mime_type_t *srctype,
+			             cups_array_t *filters, const char *infile,
+		                     const char *outfile, const char *ppdfile,
+			             const char *printer, const char *user,
+				     const char *title, int num_options,
+			             cups_option_t *options);
+static void		get_job_file(const char *job);
+static int		open_pipe(int *fds);
+static int		read_cupsd_conf(const char *filename);
+static void		set_string(char **s, const char *val);
+static void		sighandler(int sig);
+static void		usage(const char *command, const char *opt);
 
 
 /*
@@ -95,7 +115,10 @@ main(int  argc,				/* I - Number of command-line args */
 {
   int		i;			/* Looping vars */
   const char	*command,		/* Command name */
-		*opt;			/* Current option */
+		*opt,			/* Current option */
+		*printer;		/* Printer name */
+  mime_type_t	*printer_type,		/* Printer MIME type */
+		*prefilter_type;	/* Printer prefilter MIME type */
   char		*srctype,		/* Source type */
 		*dsttype,		/* Destination type */
 		super[MIME_MAX_SUPER],	/* Super-type name */
@@ -103,6 +126,7 @@ main(int  argc,				/* I - Number of command-line args */
   int		compression;		/* Compression of file */
   int		cost;			/* Cost of filters */
   mime_t	*mime;			/* MIME database */
+  char		mimedir[1024];		/* MIME directory */
   char		*infile,		/* File to filter */
 		*outfile;		/* File to create */
   char		cupsdconf[1024];	/* cupsd.conf file */
@@ -115,7 +139,8 @@ main(int  argc,				/* I - Number of command-line args */
   const char	*ppdfile;		/* PPD file */
   const char	*title,			/* Title string */
 		*user;			/* Username */
-  int		removeppd,		/* Remove PPD file */
+  int		all_filters,		/* Use all filters */
+		removeppd,		/* Remove PPD file */
 		removeinfile;		/* Remove input file */
   int		status;			/* Execution status */
 
@@ -129,6 +154,7 @@ main(int  argc,				/* I - Number of command-line args */
   else
     command = argv[0];
 
+  printer      = !strcmp(command, "convert") ? "tofile" : "cupsfilter";
   mime         = NULL;
   srctype      = NULL;
   compression  = 0;
@@ -140,6 +166,7 @@ main(int  argc,				/* I - Number of command-line args */
   ppdfile      = NULL;
   title        = NULL;
   user         = cupsUser();
+  all_filters  = 0;
   removeppd    = 0;
   removeinfile = 0;
 
@@ -190,10 +217,22 @@ main(int  argc,				/* I - Number of command-line args */
 	        usage(command, opt);
 	      break;
 
+          case 'd' : /* Specify the real printer name */
+	      i ++;
+	      if (i < argc)
+	        printer = argv[i];
+	      else
+	        usage(command, opt);
+	      break;
+
 	  case 'D' : /* Delete input file after conversion */
 	      removeinfile = 1;
 	      break;
 
+          case 'e' : /* Use every filter from the PPD file */
+	      all_filters = 1;
+	      break;
+
           case 'f' : /* Specify input file... */
 	      i ++;
 	      if (i < argc && !infile)
@@ -215,7 +254,21 @@ main(int  argc,				/* I - Number of command-line args */
 	        usage(command, opt);
 	      break;
 
-          case 'j' : /* Specify destination MIME type... */
+          case 'j' : /* Get job file or specify destination MIME type... */
+              if (strcmp(command, "convert"))
+	      {
+	        i ++;
+		if (i < argc)
+		{
+		  get_job_file(argv[i]);
+		  infile = TempFile;
+		}
+		else
+		  usage(command, opt);
+
+                break;
+	      }
+
           case 'm' : /* Specify destination MIME type... */
 	      i ++;
 	      if (i < argc)
@@ -276,7 +329,7 @@ main(int  argc,				/* I - Number of command-line args */
 	      break;
 
 	  case 'u' : /* Delete PPD file after conversion */
-	      removeinfile = 1;
+	      removeppd = 1;
 	      break;
 
           case 'U' : /* Specify username... */
@@ -300,14 +353,14 @@ main(int  argc,				/* I - Number of command-line args */
       {
 	_cupsLangPuts(stderr,
 		      _("convert: Use the -f option to specify a file to "
-		        "convert.\n"));
+		        "convert."));
 	usage(command, NULL);
       }
     }
     else
     {
       _cupsLangPuts(stderr,
-                    _("cupsfilter: Only one filename can be specified!\n"));
+                    _("cupsfilter: Only one filename can be specified."));
       usage(command, NULL);
     }
 
@@ -331,14 +384,30 @@ main(int  argc,				/* I - Number of command-line args */
   if (read_cupsd_conf(cupsdconf))
     return (1);
 
-  if ((mime = mimeLoad(ServerRoot, Path)) == NULL)
+  snprintf(mimedir, sizeof(mimedir), "%s/mime", DataDir);
+
+  mime = mimeLoadTypes(NULL, mimedir);
+  mime = mimeLoadTypes(mime, ServerRoot);
+  mime = mimeLoadFilters(mime, mimedir, Path);
+  mime = mimeLoadFilters(mime, ServerRoot, Path);
+
+  if (!mime)
   {
     _cupsLangPrintf(stderr,
-                    _("%s: Unable to read MIME database from \"%s\"!\n"),
-		    command, ServerRoot);
+                    _("%s: Unable to read MIME database from \"%s\" or "
+		      "\"%s\"."),
+		    command, mimedir, ServerRoot);
     return (1);
   }
 
+  prefilter_type = NULL;
+
+  if (all_filters)
+    printer_type = add_printer_filters(command, mime, printer, ppdfile,
+				       &prefilter_type);
+  else
+    printer_type   = mimeType(mime, "application", "vnd.cups-postscript");
+
  /*
   * Get the source and destination types...
   */
@@ -349,7 +418,7 @@ main(int  argc,				/* I - Number of command-line args */
     if ((src = mimeType(mime, super, type)) == NULL)
     {
       _cupsLangPrintf(stderr,
-		      _("%s: Unknown source MIME type %s/%s!\n"),
+		      _("%s: Unknown source MIME type %s/%s."),
 		      command, super, type);
       return (1);
     }
@@ -357,16 +426,18 @@ main(int  argc,				/* I - Number of command-line args */
   else if ((src = mimeFileType(mime, infile, infile, &compression)) == NULL)
   {
     _cupsLangPrintf(stderr,
-                    _("%s: Unable to determine MIME type of \"%s\"!\n"),
+                    _("%s: Unable to determine MIME type of \"%s\"."),
 		    command, infile);
     return (1);
   }
 
   sscanf(dsttype, "%15[^/]/%255s", super, type);
-  if ((dst = mimeType(mime, super, type)) == NULL)
+  if (!_cups_strcasecmp(super, "printer"))
+    dst = printer_type;
+  else if ((dst = mimeType(mime, super, type)) == NULL)
   {
     _cupsLangPrintf(stderr,
-                    _("%s: Unknown destination MIME type %s/%s!\n"),
+                    _("%s: Unknown destination MIME type %s/%s."),
 		    command, super, type);
     return (1);
   }
@@ -383,29 +454,60 @@ main(int  argc,				/* I - Number of command-line args */
 
     filters = cupsArrayNew(NULL, NULL);
     cupsArrayAdd(filters, &GZIPFilter);
+    GZIPFilter.src = src;
+    GZIPFilter.dst = dst;
   }
   else if ((filters = mimeFilter(mime, src, dst, &cost)) == NULL)
   {
     _cupsLangPrintf(stderr,
-                    _("%s: No filter to convert from %s/%s to %s/%s!\n"),
+                    _("%s: No filter to convert from %s/%s to %s/%s."),
 		    command, src->super, src->type, dst->super, dst->type);
     return (1);
   }
   else if (compression)
     cupsArrayInsert(filters, &GZIPFilter);
 
+  if (prefilter_type)
+  {
+   /*
+    * Add pre-filters...
+    */
+
+    mime_filter_t	*filter,	/* Current filter */
+			*prefilter;	/* Current pre-filter */
+    cups_array_t	*prefilters = cupsArrayNew(NULL, NULL);
+					/* New filters array */
+
+
+    for (filter = (mime_filter_t *)cupsArrayFirst(filters);
+	 filter;
+	 filter = (mime_filter_t *)cupsArrayNext(filters))
+    {
+      if ((prefilter = mimeFilterLookup(mime, filter->src,
+                                        prefilter_type)) != NULL)
+	cupsArrayAdd(prefilters, prefilter);
+
+      cupsArrayAdd(prefilters, filter);
+    }
+
+    cupsArrayDelete(filters);
+    filters = prefilters;
+  }
+
  /*
   * Do it!
   */
 
-  status = exec_filters(filters, infile, outfile, ppdfile,
-                        !strcmp(command, "convert") ? "tofile" : "cupsfilter",
-			user, title, num_options, options);
+  status = exec_filters(src, filters, infile, outfile, ppdfile, printer, user,
+                        title, num_options, options);
 
  /*
   * Remove files as needed, then exit...
   */
 
+  if (TempFile[0])
+    unlink(TempFile);
+
   if (removeppd && ppdfile)
     unlink(ppdfile);
 
@@ -416,6 +518,208 @@ main(int  argc,				/* I - Number of command-line args */
 }
 
 
+/*
+ * 'add_printer_filter()' - Add a single filters from a PPD file.
+ */
+
+static void
+add_printer_filter(
+    const char  *command,		/* I - Command name */
+    mime_t      *mime,			/* I - MIME database */
+    mime_type_t *filtertype,		/* I - Printer or prefilter MIME type */
+    const char  *filter)		/* I - Filter to add */
+{
+  char		super[MIME_MAX_SUPER],	/* Super-type for filter */
+		type[MIME_MAX_TYPE],	/* Type for filter */
+		dsuper[MIME_MAX_SUPER],	/* Destination super-type for filter */
+		dtype[MIME_MAX_TYPE],	/* Destination type for filter */
+		dest[MIME_MAX_SUPER + MIME_MAX_TYPE + 2],
+					/* Destination super/type */
+		program[1024];		/* Program/filter name */
+  int		cost;			/* Cost of filter */
+  size_t	maxsize = 0;		/* Maximum supported file size */
+  mime_type_t	*temptype,		/* MIME type looping var */
+		*desttype;		/* Destination MIME type */
+  mime_filter_t	*filterptr;		/* MIME filter */
+
+
+ /*
+  * Parse the filter string; it should be in one of the following formats:
+  *
+  *     source/type cost program
+  *     source/type cost maxsize(nnnn) program
+  *     source/type dest/type cost program
+  *     source/type dest/type cost maxsize(nnnn) program
+  */
+
+  if (sscanf(filter, "%15[^/]/%255s%*[ \t]%15[^/]/%255s%d%*[ \t]%1023[^\n]",
+             super, type, dsuper, dtype, &cost, program) == 6)
+  {
+    snprintf(dest, sizeof(dest), "%s/%s/%s", filtertype->type, dsuper, dtype);
+
+    if ((desttype = mimeType(mime, "printer", dest)) == NULL)
+      desttype = mimeAddType(mime, "printer", dest);
+  }
+  else
+  {
+    if (sscanf(filter, "%15[^/]/%255s%d%*[ \t]%1023[^\n]", super, type, &cost,
+               program) == 4)
+    {
+      desttype = filtertype;
+    }
+    else
+    {
+      _cupsLangPrintf(stderr, _("%s: Invalid filter string \"%s\"."), command,
+		      filter);
+      return;
+    }
+  }
+
+  if (!strncmp(program, "maxsize(", 8))
+  {
+    char	*ptr;			/* Pointer into maxsize(nnnn) program */
+
+    maxsize = strtoll(program + 8, &ptr, 10);
+
+    if (*ptr != ')')
+    {
+      printf("testmime: Invalid filter string \"%s\".\n", filter);
+      return;
+    }
+
+    ptr ++;
+    while (_cups_isspace(*ptr))
+      ptr ++;
+
+    _cups_strcpy(program, ptr);
+  }
+
+ /*
+  * See if the filter program exists; if not, stop the printer and flag
+  * the error!
+  */
+
+  if (strcmp(program, "-"))
+  {
+    char filename[1024];		/* Full path to program */
+
+    if (program[0] == '/')
+      strlcpy(filename, program, sizeof(filename));
+    else
+      snprintf(filename, sizeof(filename), "%s/filter/%s", ServerBin, program);
+
+    if (_cupsFileCheck(filename, _CUPS_FILE_CHECK_PROGRAM, !geteuid(), check_cb,
+                       (void *)command))
+      return;
+  }
+
+ /*
+  * Add the filter to the MIME database, supporting wildcards as needed...
+  */
+
+  for (temptype = mimeFirstType(mime);
+       temptype;
+       temptype = mimeNextType(mime))
+    if (((super[0] == '*' && _cups_strcasecmp(temptype->super, "printer")) ||
+         !_cups_strcasecmp(temptype->super, super)) &&
+        (type[0] == '*' || !_cups_strcasecmp(temptype->type, type)))
+    {
+      if (desttype != filtertype)
+      {
+        filterptr = mimeAddFilter(mime, temptype, desttype, cost, program);
+
+        if (!mimeFilterLookup(mime, desttype, filtertype))
+          mimeAddFilter(mime, desttype, filtertype, 0, "-");
+      }
+      else
+        filterptr = mimeAddFilter(mime, temptype, filtertype, cost, program);
+
+      if (filterptr)
+	filterptr->maxsize = maxsize;
+    }
+}
+
+
+/*
+ * 'add_printer_filters()' - Add filters from a PPD file.
+ */
+
+static mime_type_t *			/* O - Printer type or NULL on error */
+add_printer_filters(
+    const char  *command,		/* I - Command name */
+    mime_t      *mime,			/* I - MIME database */
+    const char  *printer,		/* I - Printer name */
+    const char  *ppdfile,		/* I - PPD file */
+    mime_type_t **prefilter_type)	/* O - Prefilter type */
+{
+  ppd_file_t	*ppd;			/* PPD file data */
+  _ppd_cache_t	*pc;			/* Cache data for PPD */
+  const char	*value;			/* Filter definition value */
+  mime_type_t	*printer_type;		/* Printer filter type */
+
+
+  if ((ppd = ppdOpenFile(ppdfile)) == NULL)
+  {
+    ppd_status_t	status;		/* PPD load status */
+    int			linenum;	/* Line number */
+
+    status = ppdLastError(&linenum);
+    _cupsLangPrintf(stderr, _("%s: Unable to open PPD file: %s on line %d."),
+                    command, ppdErrorString(status), linenum);
+    return (NULL);
+  }
+
+  pc = _ppdCacheCreateWithPPD(ppd);
+  if (!pc)
+    return (NULL);
+
+  printer_type    = mimeAddType(mime, "printer", printer);
+  *prefilter_type = NULL;
+
+  if (pc->filters)
+  {
+    for (value = (const char *)cupsArrayFirst(pc->filters);
+         value;
+         value = (const char *)cupsArrayNext(pc->filters))
+      add_printer_filter(command, mime, printer_type, value);
+  }
+  else
+  {
+    add_printer_filter(command, mime, printer_type,
+                       "application/vnd.cups-raw 0 -");
+    add_printer_filter(command, mime, printer_type,
+                       "application/vnd.cups-postscript 0 -");
+  }
+
+  if (pc->prefilters)
+  {
+    *prefilter_type = mimeAddType(mime, "prefilter", printer);
+
+    for (value = (const char *)cupsArrayFirst(pc->prefilters);
+         value;
+         value = (const char *)cupsArrayNext(pc->prefilters))
+      add_printer_filter(command, mime, *prefilter_type, value);
+  }
+
+  return (printer_type);
+}
+
+
+/*
+ * 'check_cb()' - Callback function for _cupsFileCheck.
+ */
+
+static void
+check_cb(void              *context,	/* I - Context (command name) */
+         _cups_fc_result_t result,	/* I - Result of check */
+	 const char        *message)	/* I - Localized message */
+{
+  (void)result;
+
+  _cupsLangPrintf(stderr, _("%s: %s"), (char *)context, message);
+}
+
+
 /*
  * 'compare_pids()' - Compare two filter PIDs...
  */
@@ -502,7 +806,8 @@ exec_filter(const char *filter,		/* I - Filter to execute */
 	    int        infd,		/* I - Stdin file descriptor */
 	    int        outfd)		/* I - Stdout file descriptor */
 {
-  int		pid;			/* Process ID */
+  int		pid,			/* Process ID */
+		fd;			/* Temporary file descriptor */
 #if defined(__APPLE__)
   char		processPath[1024],	/* CFProcessPath environment variable */
 		linkpath[1024];		/* Link path for symlinks... */
@@ -510,7 +815,7 @@ exec_filter(const char *filter,		/* I - Filter to execute */
 
 
  /*
-  * Add special voodoo magic for MacOS X - this allows MacOS X 
+  * Add special voodoo magic for MacOS X - this allows MacOS X
   * programs to access their bundle resources properly...
   */
 
@@ -546,28 +851,40 @@ exec_filter(const char *filter,		/* I - Filter to execute */
 
     if (infd != 0)
     {
-      close(0);
+      if (infd < 0)
+        infd = open("/dev/null", O_RDONLY);
+
       if (infd > 0)
-        dup(infd);
-      else
-        open("/dev/null", O_RDONLY);
+      {
+        dup2(infd, 0);
+	close(infd);
+      }
     }
 
     if (outfd != 1)
     {
-      close(1);
-      if (outfd > 0)
-	dup(outfd);
-      else
-        open("/dev/null", O_WRONLY);
+      if (outfd < 0)
+        outfd = open("/dev/null", O_WRONLY);
+
+      if (outfd > 1)
+      {
+	dup2(outfd, 1);
+	close(outfd);
+      }
     }
 
-    close(3);
-    open("/dev/null", O_RDWR);
+    if ((fd = open("/dev/null", O_RDWR)) > 3)
+    {
+      dup2(fd, 3);
+      close(fd);
+    }
     fcntl(3, F_SETFL, O_NDELAY);
 
-    close(4);
-    open("/dev/null", O_RDWR);
+    if ((fd = open("/dev/null", O_RDWR)) > 4)
+    {
+      dup2(fd, 4);
+      close(fd);
+    }
     fcntl(4, F_SETFL, O_NDELAY);
 
    /*
@@ -590,7 +907,8 @@ exec_filter(const char *filter,		/* I - Filter to execute */
  */
 
 static int				/* O - 0 on success, 1 on error */
-exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
+exec_filters(mime_type_t   *srctype,	/* I - Source type */
+             cups_array_t  *filters,	/* I - Array of filters to run */
              const char    *infile,	/* I - File to filter */
 	     const char    *outfile,	/* I - File to create */
 	     const char    *ppdfile,	/* I - PPD file, if any */
@@ -602,9 +920,10 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
 {
   int		i;			/* Looping var */
   const char	*argv[8],		/* Command-line arguments */
-		*envp[11],		/* Environment variables */
+		*envp[15],		/* Environment variables */
 		*temp;			/* Temporary string */
   char		*optstr,		/* Filter options */
+		content_type[1024],	/* CONTENT_TYPE */
 		cups_datadir[1024],	/* CUPS_DATADIR */
 		cups_fontpath[1024],	/* CUPS_FONTPATH */
 		cups_serverbin[1024],	/* CUPS_SERVERBIN */
@@ -612,7 +931,10 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
 		lang[1024],		/* LANG */
 		path[1024],		/* PATH */
 		ppd[1024],		/* PPD */
-		rip_cache[1024],	/* RIP_CACHE */
+		printer_info[255],	/* PRINTER_INFO env variable */
+		printer_location[255],	/* PRINTER_LOCATION env variable */
+		printer_name[255],	/* PRINTER env variable */
+		rip_max_cache[1024],	/* RIP_MAX_CACHE */
 		userenv[1024],		/* USER */
 		program[1024];		/* Program to run */
   mime_filter_t	*filter,		/* Current filter */
@@ -625,6 +947,7 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
   cups_array_t	*pids;			/* Executed filters array */
   mime_filter_t	key;			/* Search key for filters */
   cups_lang_t	*language;		/* Current language */
+  cups_dest_t	*dest;			/* Destination information */
 
 
  /*
@@ -633,6 +956,8 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
 
   optstr = escape_options(num_options, options);
 
+  snprintf(content_type, sizeof(content_type), "CONTENT_TYPE=%s/%s",
+           srctype->super, srctype->type);
   snprintf(cups_datadir, sizeof(cups_datadir), "CUPS_DATADIR=%s", DataDir);
   snprintf(cups_fontpath, sizeof(cups_fontpath), "CUPS_FONTPATH=%s", FontPath);
   snprintf(cups_serverbin, sizeof(cups_serverbin), "CUPS_SERVERBIN=%s",
@@ -661,9 +986,37 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
 #else
     snprintf(ppd, sizeof(ppd), "PPD=%s/model/laserjet.ppd", DataDir);
 #endif /* __APPLE__ */
-  snprintf(rip_cache, sizeof(rip_cache), "RIP_CACHE=%s", RIPCache);
+  snprintf(rip_max_cache, sizeof(rip_max_cache), "RIP_MAX_CACHE=%s", RIPCache);
   snprintf(userenv, sizeof(userenv), "USER=%s", user);
 
+  if (printer &&
+      (dest = cupsGetNamedDest(CUPS_HTTP_DEFAULT, printer, NULL)) != NULL)
+  {
+    if ((temp = cupsGetOption("printer-info", dest->num_options,
+                              dest->options)) != NULL)
+      snprintf(printer_info, sizeof(printer_info), "PRINTER_INFO=%s", temp);
+    else
+      snprintf(printer_info, sizeof(printer_info), "PRINTER_INFO=%s", printer);
+
+    if ((temp = cupsGetOption("printer-location", dest->num_options,
+                              dest->options)) != NULL)
+      snprintf(printer_location, sizeof(printer_location),
+               "PRINTER_LOCATION=%s", temp);
+    else
+      strlcpy(printer_location, "PRINTER_LOCATION=Unknown",
+              sizeof(printer_location));
+  }
+  else
+  {
+    snprintf(printer_info, sizeof(printer_info), "PRINTER_INFO=%s",
+             printer ? printer : "Unknown");
+    strlcpy(printer_location, "PRINTER_LOCATION=Unknown",
+            sizeof(printer_location));
+  }
+
+  snprintf(printer_name, sizeof(printer_name), "PRINTER=%s",
+	   printer ? printer : "Unknown");
+
   argv[0] = (char *)printer;
   argv[1] = "1";
   argv[2] = user;
@@ -677,16 +1030,20 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
     argv[4] = "1";
 
   envp[0]  = "<CFProcessPath>";
-  envp[1]  = cups_datadir;
-  envp[2]  = cups_fontpath;
-  envp[3]  = cups_serverbin;
-  envp[4]  = cups_serverroot;
-  envp[5]  = lang;
-  envp[6]  = path;
-  envp[7]  = ppd;
-  envp[8]  = rip_cache;
-  envp[9]  = userenv;
-  envp[10] = NULL;
+  envp[1]  = content_type;
+  envp[2]  = cups_datadir;
+  envp[3]  = cups_fontpath;
+  envp[4]  = cups_serverbin;
+  envp[5]  = cups_serverroot;
+  envp[6]  = lang;
+  envp[7]  = path;
+  envp[8]  = ppd;
+  envp[9]  = printer_info;
+  envp[10] = printer_location;
+  envp[11] = printer_name;
+  envp[12] = rip_max_cache;
+  envp[13] = userenv;
+  envp[14] = NULL;
 
   for (i = 0; argv[i]; i ++)
     fprintf(stderr, "DEBUG: argv[%d]=\"%s\"\n", i, argv[i]);
@@ -794,10 +1151,10 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
       if (status)
       {
 	if (WIFEXITED(status))
-	  fprintf(stderr, "ERROR: %s (PID %d) stopped with status %d!\n",
+	  fprintf(stderr, "ERROR: %s (PID %d) stopped with status %d\n",
 		  filter->filter, pid, WEXITSTATUS(status));
 	else
-	  fprintf(stderr, "ERROR: %s (PID %d) crashed on signal %d!\n",
+	  fprintf(stderr, "ERROR: %s (PID %d) crashed on signal %d\n",
 		  filter->filter, pid, WTERMSIG(status));
 
         retval = 1;
@@ -814,11 +1171,101 @@ exec_filters(cups_array_t  *filters,	/* I - Array of filters to run */
 }
 
 
+/*
+ * 'get_job_file()' - Get the specified job file.
+ */
+
+static void
+get_job_file(const char *job)		/* I - Job ID */
+{
+  long		jobid,			/* Job ID */
+		docnum;			/* Document number */
+  const char	*jobptr;		/* Pointer into job ID string */
+  char		uri[1024];		/* job-uri */
+  http_t	*http;			/* Connection to server */
+  ipp_t		*request;		/* Request data */
+  int		tempfd;			/* Temporary file */
+
+
+ /*
+  * Get the job ID and document number, if any...
+  */
+
+  if ((jobptr = strrchr(job, '-')) != NULL)
+    jobptr ++;
+  else
+    jobptr = job;
+
+  jobid = strtol(jobptr, (char **)&jobptr, 10);
+
+  if (*jobptr == ',')
+    docnum = strtol(jobptr + 1, NULL, 10);
+  else
+    docnum = 1;
+
+  if (jobid < 1 || jobid > INT_MAX)
+  {
+    _cupsLangPrintf(stderr, _("cupsfilter: Invalid job ID %d."), (int)jobid);
+    exit(1);
+  }
+
+  if (docnum < 1 || docnum > INT_MAX)
+  {
+    _cupsLangPrintf(stderr, _("cupsfilter: Invalid document number %d."),
+                    (int)docnum);
+    exit(1);
+  }
+
+ /*
+  * Ask the server for the document file...
+  */
+
+  if ((http = httpConnectEncrypt(cupsServer(), ippPort(),
+                                 cupsEncryption())) == NULL)
+  {
+    _cupsLangPrintf(stderr, _("%s: Unable to connect to server."),
+                    "cupsfilter");
+    exit(1);
+  }
+
+  request = ippNewRequest(CUPS_GET_DOCUMENT);
+
+  snprintf(uri, sizeof(uri), "ipp://localhost/jobs/%d", (int)jobid);
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, uri);
+  ippAddInteger(request, IPP_TAG_OPERATION, IPP_TAG_INTEGER, "document-number",
+                (int)docnum);
+
+  if ((tempfd = cupsTempFd(TempFile, sizeof(TempFile))) == -1)
+  {
+    _cupsLangPrintError("ERROR", _("Unable to create temporary file"));
+    httpClose(http);
+    exit(1);
+  }
+
+  signal(SIGTERM, sighandler);
+
+  ippDelete(cupsDoIORequest(http, request, "/", -1, tempfd));
+
+  close(tempfd);
+
+  httpClose(http);
+
+  if (cupsLastError() != IPP_OK)
+  {
+    _cupsLangPrintf(stderr, _("cupsfilter: Unable to get job file - %s"),
+                    cupsLastErrorString());
+    unlink(TempFile);
+    exit(1);
+  }
+}
+
+
 /*
  * 'open_pipe()' - Create a pipe which is closed on exec.
  */
 
-int					/* O - 0 on success, -1 on error */
+static int				/* O - 0 on success, -1 on error */
 open_pipe(int *fds)			/* O - Pipe file descriptors (2) */
 {
  /*
@@ -891,7 +1338,7 @@ read_cupsd_conf(const char *filename)	/* I - File to read */
   else
     set_string(&FontPath, CUPS_FONTPATH);
 
-  set_string(&RIPCache, "8m");
+  set_string(&RIPCache, "128m");
 
   if ((temp = getenv("CUPS_SERVERBIN")) != NULL)
     set_string(&ServerBin, temp);
@@ -912,15 +1359,15 @@ read_cupsd_conf(const char *filename)	/* I - File to read */
 
     while (cupsFileGetConf(fp, line, sizeof(line), &ptr, &linenum))
     {
-      if (!strcasecmp(line, "DataDir"))
+      if (!_cups_strcasecmp(line, "DataDir"))
         set_string(&DataDir, ptr);
-      else if (!strcasecmp(line, "FontPath"))
+      else if (!_cups_strcasecmp(line, "FontPath"))
         set_string(&FontPath, ptr);
-      else if (!strcasecmp(line, "RIPCache"))
+      else if (!_cups_strcasecmp(line, "RIPCache"))
         set_string(&RIPCache, ptr);
-      else if (!strcasecmp(line, "ServerBin"))
+      else if (!_cups_strcasecmp(line, "ServerBin"))
         set_string(&ServerBin, ptr);
-      else if (!strcasecmp(line, "ServerRoot"))
+      else if (!_cups_strcasecmp(line, "ServerRoot"))
         set_string(&ServerRoot, ptr);
     }
 
@@ -951,6 +1398,28 @@ set_string(char       **s,		/* O - Copy of string */
 }
 
 
+/*
+ * 'sighandler()' - Signal catcher for when we print from stdin...
+ */
+
+static void
+sighandler(int s)			/* I - Signal number */
+{
+ /*
+  * Remove the temporary file we're using to print a job file...
+  */
+
+  if (TempFile[0])
+    unlink(TempFile);
+
+ /*
+  * Exit...
+  */
+
+  exit(s);
+}
+
+
 /*
  * 'usage()' - Show program usage...
  */
@@ -960,41 +1429,66 @@ usage(const char *command,		/* I - Command name */
       const char *opt)			/* I - Incorrect option, if any */
 {
   if (opt)
-    _cupsLangPrintf(stderr, _("%s: Unknown option '%c'!\n"), command, *opt);
+    _cupsLangPrintf(stderr, _("%s: Unknown option \"%c\"."), command, *opt);
 
   if (!strcmp(command, "cupsfilter"))
-    _cupsLangPuts(stdout,
-		  _("Usage: cupsfilter -m mime/type [ options ] filename\n"
-		    "\n"
-		    "Options:\n"
-		    "\n"
-		    "  -c cupsd.conf    Set cupsd.conf file to use\n"
-		    "  -n copies        Set number of copies\n"
-		    "  -o name=value    Set option(s)\n"
-		    "  -p filename.ppd  Set PPD file\n"
-		    "  -t title         Set title\n"));
+  {
+    _cupsLangPuts(stdout, _("Usage: cupsfilter [ options ] filename"));
+    _cupsLangPuts(stdout, _("Options:"));
+    _cupsLangPuts(stdout, _("  -D                      Remove the input file "
+                            "when finished."));
+    _cupsLangPuts(stdout, _("  -P filename.ppd         Set PPD file."));
+    _cupsLangPuts(stdout, _("  -U username             Set username for job."));
+    _cupsLangPuts(stdout, _("  -c cupsd.conf           Set cupsd.conf file to "
+                            "use."));
+    _cupsLangPuts(stdout, _("  -d printer              Use the named "
+                            "printer."));
+    _cupsLangPuts(stdout, _("  -e                      Use every filter from "
+                            "the PPD file."));
+    _cupsLangPuts(stdout, _("  -i mime/type            Set input MIME type "
+                            "(otherwise auto-typed)."));
+    _cupsLangPuts(stdout, _("  -j job-id[,N]           Filter file N from the "
+                            "specified job (default is file 1)."));
+    _cupsLangPuts(stdout, _("  -m mime/type            Set output MIME type "
+			    "(otherwise application/pdf)."));
+    _cupsLangPuts(stdout, _("  -n copies               Set number of copies."));
+    _cupsLangPuts(stdout, _("  -o name=value           Set option(s)."));
+    _cupsLangPuts(stdout, _("  -p filename.ppd         Set PPD file."));
+    _cupsLangPuts(stdout, _("  -t title                Set title."));
+    _cupsLangPuts(stdout, _("  -u                      Remove the PPD file "
+                            "when finished."));
+  }
   else
-    _cupsLangPuts(stdout,
-		  _("Usage: convert [ options ]\n"
-		    "\n"
-		    "Options:\n"
-		    "\n"
-		    "  -f filename          Set file to be converted (otherwise stdin)\n"
-		    "  -o filename          Set file to be generated (otherwise stdout)\n"
-		    "  -i mime/type         Set input MIME type (otherwise auto-typed)\n"
-		    "  -j mime/type         Set output MIME type (otherwise application/pdf)\n"
-		    "  -P filename.ppd      Set PPD file\n"
-		    "  -a 'name=value ...'  Set option(s)\n"
-		    "  -U username          Set username for job\n"
-		    "  -J title             Set title\n"
-		    "  -c copies            Set number of copies\n"
-		    "  -u                   Remove the PPD file when finished\n"
-		    "  -D                   Remove the input file when finished\n"));
+  {
+    _cupsLangPuts(stdout, _("Usage: convert [ options ]"));
+    _cupsLangPuts(stdout, _("Options:"));
+    _cupsLangPuts(stdout, _("  -D                      Remove the input file "
+                            "when finished."));
+    _cupsLangPuts(stdout, _("  -J title                Set title."));
+    _cupsLangPuts(stdout, _("  -P filename.ppd         Set PPD file."));
+    _cupsLangPuts(stdout, _("  -U username             Set username for job."));
+    _cupsLangPuts(stdout, _("  -a 'name=value ...'     Set option(s)."));
+    _cupsLangPuts(stdout, _("  -c copies               Set number of copies."));
+    _cupsLangPuts(stdout, _("  -d printer              Use the named "
+                            "printer."));
+    _cupsLangPuts(stdout, _("  -e                      Use every filter from "
+                            "the PPD file."));
+    _cupsLangPuts(stdout, _("  -f filename             Set file to be "
+                            "converted (otherwise stdin)."));
+    _cupsLangPuts(stdout, _("  -i mime/type            Set input MIME type "
+                            "(otherwise auto-typed)."));
+    _cupsLangPuts(stdout, _("  -j mime/type            Set output MIME type "
+			    "(otherwise application/pdf)."));
+    _cupsLangPuts(stdout, _("  -o filename             Set file to be "
+                            "generated (otherwise stdout)."));
+    _cupsLangPuts(stdout, _("  -u                      Remove the PPD file "
+                            "when finished."));
+  }
 
   exit(1);
 }
 
 
 /*
- * End of "$Id: cupsfilter.c 7864 2008-08-26 03:43:28Z mike $".
+ * End of "$Id: cupsfilter.c 9862 2011-08-03 02:44:09Z mike $".
  */
