@@ -1,9 +1,9 @@
 /*
- * "$Id: main.c 8657 2009-05-18 15:44:04Z mike $"
+ * "$Id: main.c 9783 2011-05-18 20:44:16Z mike $"
  *
- *   Scheduler main loop for the Common UNIX Printing System (CUPS).
+ *   Main loop for the CUPS scheduler.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -14,25 +14,26 @@
  *
  * Contents:
  *
- *   main()                    - Main entry for the CUPS scheduler.
- *   cupsdClosePipe()          - Close a pipe as necessary.
- *   cupsdOpenPipe()           - Create a pipe which is closed on exec.
- *   cupsdHoldSignals()        - Hold child and termination signals.
- *   cupsdReleaseSignals()     - Release signals for delivery.
- *   cupsdSetString()          - Set a string value.
- *   cupsdSetStringf()         - Set a formatted string value.
- *   launchd_checkin()         - Check-in with launchd and collect the
- *                               listening fds.
- *   launchd_checkout()        - Check-out with launchd.
- *   parent_handler()          - Catch USR1/CHLD signals...
- *   process_children()        - Process all dead children...
- *   sigchld_handler()         - Handle 'child' signals from old processes.
- *   sighup_handler()          - Handle 'hangup' signals to reconfigure the
- *                               scheduler.
- *   sigterm_handler()         - Handle 'terminate' signals that stop the
- *                               scheduler.
- *   select_timeout()          - Calculate the select timeout value.
- *   usage()                   - Show scheduler usage.
+ *   main()                - Main entry for the CUPS scheduler.
+ *   cupsdAddString()      - Copy and add a string to an array.
+ *   cupsdCheckProcess()   - Tell the main loop to check for dead children.
+ *   cupsdClearString()    - Clear a string.
+ *   cupsdFreeStrings()    - Free an array of strings.
+ *   cupsdHoldSignals()    - Hold child and termination signals.
+ *   cupsdReleaseSignals() - Release signals for delivery.
+ *   cupsdSetString()      - Set a string value.
+ *   cupsdSetStringf()     - Set a formatted string value.
+ *   launchd_checkin()     - Check-in with launchd and collect the listening
+ *                           fds.
+ *   launchd_checkout()    - Update the launchd KeepAlive file as needed.
+ *   parent_handler()      - Catch USR1/CHLD signals...
+ *   process_children()    - Process all dead children...
+ *   select_timeout()      - Calculate the select timeout value.
+ *   sigchld_handler()     - Handle 'child' signals from old processes.
+ *   sighup_handler()      - Handle 'hangup' signals to reconfigure the
+ *                           scheduler.
+ *   sigterm_handler()     - Handle 'terminate' signals that stop the scheduler.
+ *   usage()               - Show scheduler usage.
  */
 
 /*
@@ -44,12 +45,11 @@
 #include <sys/resource.h>
 #include <syslog.h>
 #include <grp.h>
-#include <cups/dir.h>
 
 #ifdef HAVE_LAUNCH_H
 #  include <launch.h>
 #  include <libgen.h>
-#  define CUPS_KEEPALIVE	CUPS_CACHEDIR "/org.cups.cupsd"
+#  define CUPS_KEEPALIVE CUPS_CACHEDIR "/org.cups.cupsd"
 					/* Name of the launchd KeepAlive file */
 #  ifndef LAUNCH_JOBKEY_KEEPALIVE
 #    define LAUNCH_JOBKEY_KEEPALIVE "KeepAlive"
@@ -69,10 +69,6 @@
 #  include <notify.h>
 #endif /* HAVE_NOTIFY_H */
 
-#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
-#  include <dlfcn.h>
-#endif /* __APPLE__ && HAVE_DLFCN_H */
-
 
 /*
  * Local functions...
@@ -106,12 +102,6 @@ static int		dead_children = 0;
 static int		stop_scheduler = 0;
 					/* Should the scheduler stop? */
 
-#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
-static const char *PSQLibPath = "/usr/lib/libPrintServiceQuota.dylib";
-static const char *PSQLibFuncName = "PSQUpdateQuota";
-static void *PSQLibRef;			/* libPrintServiceQuota.dylib */
-#endif /* HAVE_DLFCN_H */
-
 
 /*
  * 'main()' - Main entry for the CUPS scheduler.
@@ -134,7 +124,7 @@ main(int  argc,				/* I - Number of command-line args */
 			senddoc_time,	/* Send-Document time */
 			expire_time,	/* Subscription expire time */
 			report_time,	/* Malloc/client/job report time */
-			event_time;	/* Last time an event notification was done */
+			event_time;	/* Last event notification time */
   long			timeout;	/* Timeout for cupsdDoSelect() */
   struct rlimit		limit;		/* Runtime limit */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
@@ -146,7 +136,10 @@ main(int  argc,				/* I - Number of command-line args */
 #endif /* __sgi */
   int			run_as_child = 0;
 					/* Needed for background fork/exec */
-#ifndef __APPLE__
+#ifdef __APPLE__
+  int			use_sysman = !getuid();
+					/* Use system management functions? */
+#else
   time_t		netif_time = 0;	/* Time since last network update */
 #endif /* __APPLE__ */
 #if HAVE_LAUNCHD
@@ -162,7 +155,7 @@ main(int  argc,				/* I - Number of command-line args */
 
   if (getuid() != geteuid())
   {
-    fputs("cupsd: Cannot run as a setuid program!\n", stderr);
+    fputs("cupsd: Cannot run as a setuid program\n", stderr);
     return (1);
   }
 #endif /* HAVE_GETEUID */
@@ -195,7 +188,7 @@ main(int  argc,				/* I - Number of command-line args */
 	      if (i >= argc)
 	      {
 	        _cupsLangPuts(stderr, _("cupsd: Expected config filename "
-		                        "after \"-c\" option!\n"));
+		                        "after \"-c\" option."));
 	        usage(1);
 	      }
 
@@ -226,14 +219,14 @@ main(int  argc,				/* I - Number of command-line args */
                 if ((current = malloc(1024)) == NULL)
 		{
 		  _cupsLangPuts(stderr,
-		                _("cupsd: Unable to get current directory!\n"));
+		                _("cupsd: Unable to get current directory."));
                   return (1);
 		}
 
 		if (!getcwd(current, 1024))
 		{
 		  _cupsLangPuts(stderr,
-		                _("cupsd: Unable to get current directory!\n"));
+		                _("cupsd: Unable to get current directory."));
                   free(current);
 		  return (1);
 		}
@@ -261,26 +254,46 @@ main(int  argc,				/* I - Number of command-line args */
 	      fg      = 1;
 #else
 	      _cupsLangPuts(stderr, _("cupsd: launchd(8) support not compiled "
-	                              "in, running in normal mode.\n"));
+	                              "in, running in normal mode."));
               fg = 0;
 #endif /* HAVE_LAUNCHD */
 	      break;
 
           case 'p' : /* Stop immediately for profiling */
-              puts("Warning: -p option is for internal testing use only!");
+              fputs("cupsd: -p (startup profiling) is for internal testing "
+                    "use only!\n", stderr);
 	      stop_scheduler = 1;
 	      fg             = 1;
 	      break;
 
+          case 'P' : /* Disable security profiles */
+              fputs("cupsd: -P (disable security profiles) is for internal "
+                    "testing use only!\n", stderr);
+	      UseProfiles = 0;
+	      break;
+
+#ifdef __APPLE__
+          case 'S' : /* Disable system management functions */
+              fputs("cupsd: -S (disable system management) for internal "
+                    "testing use only!\n", stderr);
+	      use_sysman = 0;
+	      break;
+#endif /* __APPLE__ */
+
+          case 't' : /* Test the cupsd.conf file... */
+	      TestConfigFile = 1;
+	      fg             = 1;
+	      break;
+
 	  default : /* Unknown option */
               _cupsLangPrintf(stderr, _("cupsd: Unknown option \"%c\" - "
-	                                "aborting!\n"), *opt);
+	                                "aborting."), *opt);
 	      usage(1);
 	      break;
 	}
     else
     {
-      _cupsLangPrintf(stderr, _("cupsd: Unknown argument \"%s\" - aborting!\n"),
+      _cupsLangPrintf(stderr, _("cupsd: Unknown argument \"%s\" - aborting."),
                       argv[i]);
       usage(1);
     }
@@ -342,13 +355,13 @@ main(int  argc,				/* I - Number of command-line args */
       }
       else if (WIFEXITED(i))
       {
-        fprintf(stderr, "cupsd: Child exited with status %d!\n",
+        fprintf(stderr, "cupsd: Child exited with status %d\n",
 	        WEXITSTATUS(i));
 	return (2);
       }
       else
       {
-        fprintf(stderr, "cupsd: Child exited on signal %d!\n", WTERMSIG(i));
+        fprintf(stderr, "cupsd: Child exited on signal %d\n", WTERMSIG(i));
 	return (3);
       }
     }
@@ -359,7 +372,7 @@ main(int  argc,				/* I - Number of command-line args */
     * parent's file descriptors to be blocking.  This is a workaround for a
     * limitation of userland libpthread on OpenBSD.
     */
-   
+
     _thread_sys_closefrom(0);
 #endif /* __OpenBSD__ */
 
@@ -472,47 +485,27 @@ main(int  argc,				/* I - Number of command-line args */
 
   if (!cupsdReadConfiguration())
   {
-    syslog(LOG_LPR, "Unable to read configuration file \'%s\' - exiting!",
-           ConfigurationFile);
+    if (TestConfigFile)
+      printf("%s contains errors\n", ConfigurationFile);
+    else
+      syslog(LOG_LPR, "Unable to read configuration file \'%s\' - exiting!",
+	     ConfigurationFile);
     return (1);
   }
+  else if (TestConfigFile)
+  {
+    printf("%s is OK\n", ConfigurationFile);
+    return (0);
+  }
+
+ /*
+  * Clean out old temp files and printer cache data.
+  */
 
   if (!strncmp(TempDir, RequestRoot, strlen(RequestRoot)))
-  {
-   /*
-    * Clean out the temporary directory...
-    */
+    cupsdCleanFiles(TempDir, NULL);
 
-    cups_dir_t		*dir;		/* Temporary directory */
-    cups_dentry_t	*dent;		/* Directory entry */
-    char		tempfile[1024];	/* Temporary filename */
-
-
-    if ((dir = cupsDirOpen(TempDir)) != NULL)
-    {
-      cupsdLogMessage(CUPSD_LOG_INFO,
-                      "Cleaning out old temporary files in \"%s\"...", TempDir);
-
-      while ((dent = cupsDirRead(dir)) != NULL)
-      {
-        snprintf(tempfile, sizeof(tempfile), "%s/%s", TempDir, dent->filename);
-
-	if (cupsdRemoveFile(tempfile))
-	  cupsdLogMessage(CUPSD_LOG_ERROR,
-	                  "Unable to remove temporary file \"%s\" - %s",
-	                  tempfile, strerror(errno));
-        else
-	  cupsdLogMessage(CUPSD_LOG_DEBUG, "Removed temporary file \"%s\"...",
-	                  tempfile);
-      }
-
-      cupsDirClose(dir);
-    }
-    else
-      cupsdLogMessage(CUPSD_LOG_ERROR,
-                      "Unable to open temporary directory \"%s\" - %s",
-                      TempDir, strerror(errno));
-  }
+  cupsdCleanFiles(CacheDir, "*.ipp");
 
 #if HAVE_LAUNCHD
   if (Launchd)
@@ -522,46 +515,16 @@ main(int  argc,				/* I - Number of command-line args */
     */
 
     launchd_checkin();
+    launchd_checkout();
   }
 #endif /* HAVE_LAUNCHD */
 
-#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
- /*
-  * Load Print Service quota enforcement library (X Server only)
-  */
-
-  PSQLibRef = dlopen(PSQLibPath, RTLD_LAZY);
-
-  if (PSQLibRef)
-    PSQUpdateQuotaProc = dlsym(PSQLibRef, PSQLibFuncName);
-#endif /* __APPLE__ && HAVE_DLFCN_H */
-
-#ifdef HAVE_GSSAPI
-#  ifdef __APPLE__
- /*
-  * If the weak-linked GSSAPI/Kerberos library is not present, don't try
-  * to use it...
-  */
-
-  if (krb5_init_context != NULL)
-#  endif /* __APPLE__ */
-
- /*
-  * Setup a Kerberos context for the scheduler to use...
-  */
-
-  if (krb5_init_context(&KerberosContext))
-  {
-    KerberosContext = NULL;
-
-    cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to initialize Kerberos context");
-  }
-#endif /* HAVE_GSSAPI */
-
  /*
   * Startup the server...
   */
 
+  httpInitialize();
+
   cupsdStartServer();
 
  /*
@@ -655,7 +618,8 @@ main(int  argc,				/* I - Number of command-line args */
   * Start power management framework...
   */
 
-  cupsdStartSystemMonitor();
+  if (use_sysman)
+    cupsdStartSystemMonitor();
 #endif /* __APPLE__ */
 
  /*
@@ -695,12 +659,6 @@ main(int  argc,				/* I - Number of command-line args */
 
   while (!stop_scheduler)
   {
-#ifdef DEBUG
-    cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                    "main: Top of loop, dead_children=%d, NeedReload=%d",
-                    dead_children, NeedReload);
-#endif /* DEBUG */
-
    /*
     * Check if there are dead children to handle...
     */
@@ -731,29 +689,21 @@ main(int  argc,				/* I - Number of command-line args */
         cupsdPauseListening();
       }
 
-     /*
-      * Check for any active jobs...
-      */
-
-      for (job = (cupsd_job_t *)cupsArrayFirst(ActiveJobs);
-	   job;
-	   job = (cupsd_job_t *)cupsArrayNext(ActiveJobs))
-        if (job->state_value == IPP_JOB_PROCESSING)
-	  break;
-
      /*
       * Restart if all clients are closed and all jobs finished, or
       * if the reload timeout has elapsed...
       */
 
       if ((cupsArrayCount(Clients) == 0 &&
-           (!job || NeedReload != RELOAD_ALL)) ||
+           (cupsArrayCount(PrintingJobs) == 0 || NeedReload != RELOAD_ALL)) ||
           (time(NULL) - ReloadTime) >= ReloadTimeout)
       {
        /*
 	* Shutdown the server...
 	*/
 
+        DoingShutdown = 1;
+
 	cupsdStopServer();
 
        /*
@@ -771,10 +721,12 @@ main(int  argc,				/* I - Number of command-line args */
 	if (Launchd)
 	{
 	 /*
-	  * If we were started by launchd get the listen sockets file descriptors...
+	  * If we were started by launchd, get the listen socket file
+	  * descriptors...
 	  */
 
 	  launchd_checkin();
+	  launchd_checkout();
 	}
 #endif /* HAVE_LAUNCHD */
 
@@ -782,6 +734,8 @@ main(int  argc,				/* I - Number of command-line args */
         * Startup the server...
         */
 
+        DoingShutdown = 0;
+
         cupsdStartServer();
 
        /*
@@ -814,10 +768,9 @@ main(int  argc,				/* I - Number of command-line args */
 
     if (timeout == 86400 && Launchd && LaunchdTimeout && !NumPolled &&
         !cupsArrayCount(ActiveJobs) &&
-	(!Browsing || 
-	 (!BrowseRemoteProtocols && 
-	  (!NumBrowsers || !BrowseLocalProtocols ||
-	   cupsArrayCount(Printers) == 0))))
+	(!Browsing ||
+	 (!BrowseRemoteProtocols &&
+	  (!BrowseLocalProtocols || !cupsArrayCount(Printers)))))
     {
       timeout		= LaunchdTimeout;
       launchd_idle_exit = 1;
@@ -881,8 +834,8 @@ main(int  argc,				/* I - Number of command-line args */
       for (p = (cupsd_printer_t *)cupsArrayFirst(Printers);
 	   p;
 	   p = (cupsd_printer_t *)cupsArrayNext(Printers))
-        cupsdLogMessage(CUPSD_LOG_EMERG, "printer[%s] %d", p->name,
-	                p->dnssd_ipp_fd);
+        cupsdLogMessage(CUPSD_LOG_EMERG, "printer[%s] reg_name=\"%s\"", p->name,
+	                p->reg_name ? p->reg_name : "(null)");
 #endif /* HAVE_DNSSD */
 
       break;
@@ -890,6 +843,27 @@ main(int  argc,				/* I - Number of command-line args */
 
     current_time = time(NULL);
 
+   /*
+    * Write dirty config/state files...
+    */
+
+    if (DirtyCleanTime && current_time >= DirtyCleanTime)
+      cupsdCleanDirty();
+
+#ifdef __APPLE__
+   /*
+    * If we are going to sleep and still have pending jobs, stop them after
+    * a period of time...
+    */
+
+    if (SleepJobs > 0 && current_time >= SleepJobs &&
+        cupsArrayCount(PrintingJobs) > 0)
+    {
+      SleepJobs = 0;
+      cupsdStopAllJobs(CUPSD_JOB_DEFAULT, 5);
+    }
+#endif /* __APPLE__ */
+
 #ifndef __APPLE__
    /*
     * Update the network interfaces once a minute...
@@ -965,6 +939,7 @@ main(int  argc,				/* I - Number of command-line args */
       browse_time = current_time;
     }
 
+#ifndef HAVE_AUTHORIZATION_H
    /*
     * Update the root certificate once every 5 minutes if we have client
     * connections...
@@ -980,6 +955,7 @@ main(int  argc,				/* I - Number of command-line args */
       cupsdDeleteCert(0);
       cupsdAddCert(0, "root", NULL);
     }
+#endif /* !HAVE_AUTHORIZATION_H */
 
    /*
     * Check for new data on the client sockets...
@@ -1022,6 +998,7 @@ main(int  argc,				/* I - Number of command-line args */
     if ((current_time - senddoc_time) >= 10)
     {
       cupsdCheckJobs();
+      cupsdCleanJobs();
       senddoc_time = current_time;
     }
 
@@ -1136,6 +1113,8 @@ main(int  argc,				/* I - Number of command-line args */
   * Close all network clients...
   */
 
+  DoingShutdown = 1;
+
   cupsdStopServer();
 
 #ifdef HAVE_LAUNCHD
@@ -1158,7 +1137,8 @@ main(int  argc,				/* I - Number of command-line args */
   * Stop monitoring system event monitoring...
   */
 
-  cupsdStopSystemMonitor();
+  if (use_sysman)
+    cupsdStopSystemMonitor();
 #endif /* __APPLE__ */
 
 #ifdef HAVE_GSSAPI
@@ -1178,19 +1158,6 @@ main(int  argc,				/* I - Number of command-line args */
     krb5_free_context(KerberosContext);
 #endif /* HAVE_GSSAPI */
 
-#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
- /* 
-  * Unload Print Service quota enforcement library (X Server only) 
-  */
-
-  PSQUpdateQuotaProc = NULL;
-  if (PSQLibRef)
-  {
-    dlclose(PSQLibRef);
-    PSQLibRef = NULL;
-  }
-#endif /* __APPLE__ && HAVE_DLFCN_H */
-
 #ifdef __sgi
  /*
   * Remove the fake IRIX lpsched lock file, but only if the existing
@@ -1210,80 +1177,35 @@ main(int  argc,				/* I - Number of command-line args */
 
 
 /*
- * 'cupsdClosePipe()' - Close a pipe as necessary.
+ * 'cupsdAddString()' - Copy and add a string to an array.
  */
 
-void
-cupsdClosePipe(int *fds)		/* I - Pipe file descriptors (2) */
+int					/* O  - 1 on success, 0 on failure */
+cupsdAddString(cups_array_t **a,	/* IO - String array */
+               const char   *s)		/* I  - String to copy and add */
 {
- /*
-  * Close file descriptors as needed...
-  */
+  if (!*a)
+    *a = cupsArrayNew3((cups_array_func_t)strcmp, NULL,
+		       (cups_ahash_func_t)NULL, 0,
+		       (cups_acopy_func_t)_cupsStrAlloc,
+		       (cups_afree_func_t)_cupsStrFree);
 
-  if (fds[0] >= 0)
-  {
-    close(fds[0]);
-    fds[0] = -1;
-  }
-
-  if (fds[1] >= 0)
-  {
-    close(fds[1]);
-    fds[1] = -1;
-  }
+  return (cupsArrayAdd(*a, (char *)s));
 }
 
 
 /*
- * 'cupsdOpenPipe()' - Create a pipe which is closed on exec.
+ * 'cupsdCheckProcess()' - Tell the main loop to check for dead children.
  */
 
-int					/* O - 0 on success, -1 on error */
-cupsdOpenPipe(int *fds)			/* O - Pipe file descriptors (2) */
+void
+cupsdCheckProcess(void)
 {
  /*
-  * Create the pipe...
+  * Flag that we have dead children...
   */
 
-  if (pipe(fds))
-  {
-    fds[0] = -1;
-    fds[1] = -1;
-
-    return (-1);
-  }
-
- /*
-  * Set the "close on exec" flag on each end of the pipe...
-  */
-
-  if (fcntl(fds[0], F_SETFD, fcntl(fds[0], F_GETFD) | FD_CLOEXEC))
-  {
-    close(fds[0]);
-    close(fds[1]);
-
-    fds[0] = -1;
-    fds[1] = -1;
-
-    return (-1);
-  }
-
-  if (fcntl(fds[1], F_SETFD, fcntl(fds[1], F_GETFD) | FD_CLOEXEC))
-  {
-    close(fds[0]);
-    close(fds[1]);
-
-    fds[0] = -1;
-    fds[1] = -1;
-
-    return (-1);
-  }
-
- /*
-  * Return 0 indicating success...
-  */
-
-  return (0);
+  dead_children = 1;
 }
 
 
@@ -1302,6 +1224,21 @@ cupsdClearString(char **s)		/* O - String value */
 }
 
 
+/*
+ * 'cupsdFreeStrings()' - Free an array of strings.
+ */
+
+void
+cupsdFreeStrings(cups_array_t **a)	/* IO - String array */
+{
+  if (*a)
+  {
+    cupsArrayDelete(*a);
+    *a = NULL;
+  }
+}
+
+
 /*
  * 'cupsdHoldSignals()' - Hold child and termination signals.
  */
@@ -1414,9 +1351,8 @@ cupsdSetStringf(char       **s,		/* O - New string */
 static void
 launchd_checkin(void)
 {
-  int			i,		/* Looping var */
-			count,		/* Numebr of listeners */
-			portnum;	/* Port number */
+  size_t		i,		/* Looping var */
+			count;		/* Number of listeners */
   launch_data_t		ld_msg,		/* Launch data message */
 			ld_resp,	/* Launch data response */
 			ld_array,	/* Launch data array */
@@ -1442,6 +1378,7 @@ launchd_checkin(void)
 		    "launchd_checkin: launch_msg(\"" LAUNCH_KEY_CHECKIN
 		    "\") IPC failure");
     exit(EXIT_FAILURE);
+    return; /* anti-compiler-warning */
   }
 
   if (launch_data_get_type(ld_resp) == LAUNCH_DATA_ERRNO)
@@ -1450,28 +1387,32 @@ launchd_checkin(void)
     cupsdLogMessage(CUPSD_LOG_ERROR, "launchd_checkin: Check-in failed: %s",
                     strerror(errno));
     exit(EXIT_FAILURE);
+    return; /* anti-compiler-warning */
   }
 
  /*
   * Get the sockets dictionary...
   */
 
-  if (!(ld_sockets = launch_data_dict_lookup(ld_resp, LAUNCH_JOBKEY_SOCKETS)))
+  if ((ld_sockets = launch_data_dict_lookup(ld_resp, LAUNCH_JOBKEY_SOCKETS))
+          == NULL)
   {
     cupsdLogMessage(CUPSD_LOG_ERROR,
                     "launchd_checkin: No sockets found to answer requests on!");
     exit(EXIT_FAILURE);
+    return; /* anti-compiler-warning */
   }
 
  /*
   * Get the array of listener sockets...
   */
 
-  if (!(ld_array = launch_data_dict_lookup(ld_sockets, "Listeners")))
+  if ((ld_array = launch_data_dict_lookup(ld_sockets, "Listeners")) == NULL)
   {
     cupsdLogMessage(CUPSD_LOG_ERROR,
                     "launchd_checkin: No sockets found to answer requests on!");
     exit(EXIT_FAILURE);
+    return; /* anti-compiler-warning */
   }
 
  /*
@@ -1488,73 +1429,65 @@ launchd_checkin(void)
       * Get the launchd file descriptor and address...
       */
 
-      tmp     = launch_data_array_get_index(ld_array, i);
-      fd      = launch_data_get_fd(tmp);
-      addrlen = sizeof(addr);
-
-      if (getsockname(fd, (struct sockaddr *)&addr, &addrlen))
+      if ((tmp = launch_data_array_get_index(ld_array, i)) != NULL)
       {
-	cupsdLogMessage(CUPSD_LOG_ERROR,
-	                "launchd_checkin: Unable to get local address - %s",
-			strerror(errno));
-	continue;
-      }
+	fd      = launch_data_get_fd(tmp);
+	addrlen = sizeof(addr);
 
-     /*
-      * Try to match the launchd socket address to one of the listeners...
-      */
-
-      for (lis = (cupsd_listener_t *)cupsArrayFirst(Listeners);
-	   lis;
-	   lis = (cupsd_listener_t *)cupsArrayNext(Listeners))
-	if (httpAddrEqual(&lis->address, &addr))
-	  break;
-
-     /*
-      * Add a new listener If there's no match...
-      */
-
-      if (lis)
-      {
-	cupsdLogMessage(CUPSD_LOG_DEBUG, 
-		"launchd_checkin: Matched existing listener %s with fd %d...",
-		httpAddrString(&(lis->address), s, sizeof(s)), fd);
-      }
-      else
-      {
-	cupsdLogMessage(CUPSD_LOG_DEBUG, 
-		"launchd_checkin: Adding new listener %s with fd %d...",
-		httpAddrString(&addr, s, sizeof(s)), fd);
-
-        if ((lis = calloc(1, sizeof(cupsd_listener_t))) == NULL)
-        {
+	if (getsockname(fd, (struct sockaddr *)&addr, &addrlen))
+	{
 	  cupsdLogMessage(CUPSD_LOG_ERROR,
-               	          "launchd_checkin: Unable to allocate listener - %s.",
-                	  strerror(errno));
-	  exit(EXIT_FAILURE);
-        }
+			  "launchd_checkin: Unable to get local address - %s",
+			  strerror(errno));
+	  continue;
+	}
 
-        cupsArrayAdd(Listeners, lis);
+       /*
+	* Try to match the launchd socket address to one of the listeners...
+	*/
 
-	memcpy(&lis->address, &addr, sizeof(lis->address));
-      }
+	for (lis = (cupsd_listener_t *)cupsArrayFirst(Listeners);
+	     lis;
+	     lis = (cupsd_listener_t *)cupsArrayNext(Listeners))
+	  if (httpAddrEqual(&lis->address, &addr))
+	    break;
 
-      lis->fd = fd;
+       /*
+	* Add a new listener If there's no match...
+	*/
+
+	if (lis)
+	{
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+		  "launchd_checkin: Matched existing listener %s with fd %d...",
+		  httpAddrString(&(lis->address), s, sizeof(s)), fd);
+	}
+	else
+	{
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+		  "launchd_checkin: Adding new listener %s with fd %d...",
+		  httpAddrString(&addr, s, sizeof(s)), fd);
+
+	  if ((lis = calloc(1, sizeof(cupsd_listener_t))) == NULL)
+	  {
+	    cupsdLogMessage(CUPSD_LOG_ERROR,
+			    "launchd_checkin: Unable to allocate listener - "
+			    "%s.", strerror(errno));
+	    exit(EXIT_FAILURE);
+	  }
+
+	  cupsArrayAdd(Listeners, lis);
+
+	  memcpy(&lis->address, &addr, sizeof(lis->address));
+	}
+
+	lis->fd = fd;
 
 #  ifdef HAVE_SSL
-      portnum = 0;
-
-#    ifdef AF_INET6
-      if (lis->address.addr.sa_family == AF_INET6)
-	portnum = ntohs(lis->address.ipv6.sin6_port);
-      else
-#    endif /* AF_INET6 */
-      if (lis->address.addr.sa_family == AF_INET)
-	portnum = ntohs(lis->address.ipv4.sin_port);
-
-      if (portnum == 443)
-	lis->encryption = HTTP_ENCRYPT_ALWAYS;
+	if (_httpAddrPort(&(lis->address)) == 443)
+	  lis->encryption = HTTP_ENCRYPT_ALWAYS;
 #  endif /* HAVE_SSL */
+      }
     }
   }
 
@@ -1575,17 +1508,18 @@ launchd_checkout(void)
 
  /*
   * Create or remove the launchd KeepAlive file based on whether
-  * there are active jobs, polling, browsing for remote printers or 
+  * there are active jobs, polling, browsing for remote printers or
   * shared printers to advertise...
   */
 
-  if ((cupsArrayCount(ActiveJobs) || NumPolled || 
-       (Browsing && 
-	(BrowseRemoteProtocols ||
-        (BrowseLocalProtocols && NumBrowsers && cupsArrayCount(Printers))))))
+  if (cupsArrayCount(ActiveJobs) || NumPolled ||
+      (Browsing &&
+       (BrowseRemoteProtocols || 
+        (BrowseLocalProtocols && cupsArrayCount(Printers)))))
   {
     cupsdLogMessage(CUPSD_LOG_DEBUG,
-                    "Creating launchd keepalive file \"" CUPS_KEEPALIVE "\"...");
+                    "Creating launchd keepalive file \"" CUPS_KEEPALIVE
+                    "\"...");
 
     if ((fd = open(CUPS_KEEPALIVE, O_RDONLY | O_CREAT | O_EXCL, S_IRUSR)) >= 0)
       close(fd);
@@ -1593,7 +1527,8 @@ launchd_checkout(void)
   else
   {
     cupsdLogMessage(CUPSD_LOG_DEBUG,
-                    "Removing launchd keepalive file \"" CUPS_KEEPALIVE "\"...");
+                    "Removing launchd keepalive file \"" CUPS_KEEPALIVE
+                    "\"...");
 
     unlink(CUPS_KEEPALIVE);
   }
@@ -1624,7 +1559,8 @@ static void
 process_children(void)
 {
   int		status;			/* Exit status of child */
-  int		pid;			/* Process ID of child */
+  int		pid,			/* Process ID of child */
+		job_id;			/* Job ID of child */
   cupsd_job_t	*job;			/* Current job */
   int		i;			/* Looping var */
   char		name[1024];		/* Process name */
@@ -1651,10 +1587,10 @@ process_children(void)
 #endif /* HAVE_WAITPID */
   {
    /*
-    * Ignore SIGTERM errors - that comes when a job is canceled...
+    * Collect the name of the process that finished...
     */
 
-    cupsdFinishProcess(pid, name, sizeof(name));
+    cupsdFinishProcess(pid, name, sizeof(name), &job_id);
 
    /*
     * Delete certificates for CGI processes...
@@ -1664,94 +1600,147 @@ process_children(void)
       cupsdDeleteCert(pid);
 
    /*
-    * Lookup the PID in the jobs list...
+    * Handle completed job filters...
     */
 
-    for (job = (cupsd_job_t *)cupsArrayFirst(ActiveJobs);
-	 job;
-	 job = (cupsd_job_t *)cupsArrayNext(ActiveJobs))
-      if (job->state_value == IPP_JOB_PROCESSING)
-      {
-	for (i = 0; job->filters[i]; i ++)
-          if (job->filters[i] == pid)
-	    break;
+    if (job_id > 0 && (job = cupsdFindJob(job_id)) != NULL)
+    {
+      for (i = 0; job->filters[i]; i ++)
+	if (job->filters[i] == pid)
+	  break;
 
-	if (job->filters[i] || job->backend == pid)
+      if (job->filters[i] || job->backend == pid)
+      {
+       /*
+	* OK, this process has gone away; what's left?
+	*/
+
+	if (job->filters[i])
+	  job->filters[i] = -pid;
+	else
+	  job->backend = -pid;
+
+	if (status && status != SIGTERM && status != SIGKILL &&
+	    status != SIGPIPE && job->status >= 0)
 	{
 	 /*
-          * OK, this process has gone away; what's left?
+	  * An error occurred; save the exit status so we know to stop
+	  * the printer or cancel the job when all of the filters finish...
+	  *
+	  * A negative status indicates that the backend failed and the
+	  * printer needs to be stopped.
 	  */
 
-          if (job->filters[i])
-	    job->filters[i] = -pid;
+	  if (job->filters[i])
+	    job->status = status;	/* Filter failed */
 	  else
-	    job->backend = -pid;
+	    job->status = -status;	/* Backend failed */
 
-          if (job->state_value == IPP_JOB_CANCELED)
-	    status = 0;			/* Ignore errors when canceling jobs */
+	  if (job->state_value == IPP_JOB_PROCESSING &&
+	      job->status_level > CUPSD_LOG_ERROR)
+	  {
+	    char	message[1024];	/* New printer-state-message */
 
-          if (status && job->status >= 0)
+
+	    job->status_level = CUPSD_LOG_ERROR;
+
+	    snprintf(message, sizeof(message), "%s failed", name);
+
+            if (job->printer)
+	    {
+	      strlcpy(job->printer->state_message, message,
+		       sizeof(job->printer->state_message));
+	    }
+
+	    if (!job->attrs)
+	      cupsdLoadJob(job);
+
+	    if (!job->printer_message && job->attrs)
+	    {
+	      if ((job->printer_message =
+	               ippFindAttribute(job->attrs, "job-printer-state-message",
+					IPP_TAG_TEXT)) == NULL)
+		job->printer_message = ippAddString(job->attrs, IPP_TAG_JOB,
+		                                    IPP_TAG_TEXT,
+						    "job-printer-state-message",
+						    NULL, NULL);
+	    }
+
+	    if (job->printer_message)
+	      cupsdSetString(&(job->printer_message->values[0].string.text),
+			     message);
+	  }
+	}
+
+       /*
+	* If this is not the last file in a job, see if all of the
+	* filters are done, and if so move to the next file.
+	*/
+
+	if (job->current_file < job->num_files && job->printer)
+	{
+	  for (i = 0; job->filters[i] < 0; i ++);
+
+	  if (!job->filters[i] &&
+	      (!job->printer->pc || !job->printer->pc->single_file ||
+	       job->backend <= 0))
 	  {
 	   /*
-	    * An error occurred; save the exit status so we know to stop
-	    * the printer or cancel the job when all of the filters finish...
-	    *
-	    * A negative status indicates that the backend failed and the
-	    * printer needs to be stopped.
+	    * Process the next file...
 	    */
 
-            if (job->filters[i])
- 	      job->status = status;	/* Filter failed */
-	    else
- 	      job->status = -status;	/* Backend failed */
-
-            if (job->printer && !(job->printer->type & CUPS_PRINTER_FAX) &&
-	        job->status_level > CUPSD_LOG_ERROR)
-	    {
-	      job->status_level = CUPSD_LOG_ERROR;
-
-              snprintf(job->printer->state_message,
-	               sizeof(job->printer->state_message), "%s failed", name);
-              cupsdAddPrinterHistory(job->printer);
-	    }
+	    cupsdContinueJob(job);
 	  }
-
+	}
+	else if (job->state_value >= IPP_JOB_CANCELED)
+	{
 	 /*
-	  * If this is not the last file in a job, see if all of the
-	  * filters are done, and if so move to the next file.
+	  * Remove the job from the active list if there are no processes still
+	  * running for it...
 	  */
 
-          if (job->current_file < job->num_files)
-	  {
-	    for (i = 0; job->filters[i] < 0; i ++);
+	  for (i = 0; job->filters[i] < 0; i++);
 
-	    if (!job->filters[i])
-	    {
-	     /*
-	      * Process the next file...
-	      */
-
-	      cupsdFinishJob(job);
-	    }
-	  }
-	  break;
+	  if (!job->filters[i] && job->backend <= 0)
+	    cupsArrayRemove(ActiveJobs, job);
 	}
       }
+    }
 
    /*
-    * Show the exit status as needed...
+    * Show the exit status as needed, ignoring SIGTERM and SIGKILL errors
+    * since they come when we kill/end a process...
     */
 
-    if (status == SIGTERM)
-      status = 0;
-
-    if (status)
+    if (status == SIGTERM || status == SIGKILL)
+    {
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "PID %d (%s) was terminated normally with signal %d.",
+                      pid, name, status);
+    }
+    else if (status == SIGPIPE)
+    {
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "PID %d (%s) did not catch or ignore signal %d.",
+                      pid, name, status);
+    }
+    else if (status)
     {
       if (WIFEXITED(status))
-	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) stopped with status %d!",
-	                pid, name, WEXITSTATUS(status));
+      {
+        int code = WEXITSTATUS(status);	/* Exit code */
+
+        if (code > 100)
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+	                  "PID %d (%s) stopped with status %d (%s)", pid, name,
+			  code, strerror(code - 100));
+	else
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+	                  "PID %d (%s) stopped with status %d.", pid, name,
+			  code);
+      }
       else
-	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) crashed on signal %d!",
+	cupsdLogMessage(CUPSD_LOG_ERROR, "PID %d (%s) crashed on signal %d.",
 	                pid, name, WTERMSIG(status));
 
       if (LogLevel < CUPSD_LOG_DEBUG)
@@ -1773,66 +1762,6 @@ process_children(void)
 }
 
 
-/*
- * 'sigchld_handler()' - Handle 'child' signals from old processes.
- */
-
-static void
-sigchld_handler(int sig)	/* I - Signal number */
-{
-  (void)sig;
-
- /*
-  * Flag that we have dead children...
-  */
-
-  dead_children = 1;
-
- /*
-  * Reset the signal handler as needed...
-  */
-
-#if !defined(HAVE_SIGSET) && !defined(HAVE_SIGACTION)
-  signal(SIGCLD, sigchld_handler);
-#endif /* !HAVE_SIGSET && !HAVE_SIGACTION */
-}
-
-
-/*
- * 'sighup_handler()' - Handle 'hangup' signals to reconfigure the scheduler.
- */
-
-static void
-sighup_handler(int sig)	/* I - Signal number */
-{
-  (void)sig;
-
-  NeedReload = RELOAD_ALL;
-  ReloadTime = time(NULL);
-
-#if !defined(HAVE_SIGSET) && !defined(HAVE_SIGACTION)
-  signal(SIGHUP, sighup_handler);
-#endif /* !HAVE_SIGSET && !HAVE_SIGACTION */
-}
-
-
-/*
- * 'sigterm_handler()' - Handle 'terminate' signals that stop the scheduler.
- */
-
-static void
-sigterm_handler(int sig)		/* I - Signal */
-{
-  (void)sig;	/* remove compiler warnings... */
-
- /*
-  * Flag that we should stop and return...
-  */
-
-  stop_scheduler = 1;
-}
-
-
 /*
  * 'select_timeout()' - Calculate the select timeout value.
  *
@@ -1878,6 +1807,18 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
   timeout = now + 86400;		/* 86400 == 1 day */
   why     = "do nothing";
 
+#ifdef __APPLE__
+ /*
+  * When going to sleep, wake up to cancel jobs that don't complete in time.
+  */
+
+  if (SleepJobs > 0 && SleepJobs < timeout)
+  {
+    timeout = SleepJobs;
+    why     = "cancel jobs before sleeping";
+  }
+#endif /* __APPLE__ */
+
  /*
   * Check whether we are accepting new connections...
   */
@@ -1954,21 +1895,41 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
     }
   }
 
+ /*
+  * Write out changes to configuration and state files...
+  */
+
+  if (DirtyCleanTime && timeout > DirtyCleanTime)
+  {
+    timeout = DirtyCleanTime;
+    why     = "write dirty config/state files";
+  }
+
  /*
   * Check for any active jobs...
   */
 
-  if (timeout > (now + 10) && ActiveJobs)
+  for (job = (cupsd_job_t *)cupsArrayFirst(ActiveJobs);
+       job;
+       job = (cupsd_job_t *)cupsArrayNext(ActiveJobs))
   {
-    for (job = (cupsd_job_t *)cupsArrayFirst(ActiveJobs);
-	 job;
-	 job = (cupsd_job_t *)cupsArrayNext(ActiveJobs))
-      if (job->state_value <= IPP_JOB_PROCESSING)
-      {
-	timeout = now + 10;
-	why     = "process active jobs";
-	break;
-      }
+    if (job->kill_time && job->kill_time < timeout)
+    {
+      timeout = job->kill_time;
+      why     = "kill unresponsive jobs";
+    }
+
+    if (job->state_value == IPP_JOB_HELD && job->hold_until < timeout)
+    {
+      timeout = job->hold_until;
+      why     = "release held jobs";
+    }
+    else if (job->state_value == IPP_JOB_PENDING && timeout > (now + 10))
+    {
+      timeout = now + 10;
+      why     = "start pending jobs";
+      break;
+    }
   }
 
 #ifdef HAVE_MALLINFO
@@ -2023,6 +1984,66 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
 }
 
 
+/*
+ * 'sigchld_handler()' - Handle 'child' signals from old processes.
+ */
+
+static void
+sigchld_handler(int sig)		/* I - Signal number */
+{
+  (void)sig;
+
+ /*
+  * Flag that we have dead children...
+  */
+
+  dead_children = 1;
+
+ /*
+  * Reset the signal handler as needed...
+  */
+
+#if !defined(HAVE_SIGSET) && !defined(HAVE_SIGACTION)
+  signal(SIGCLD, sigchld_handler);
+#endif /* !HAVE_SIGSET && !HAVE_SIGACTION */
+}
+
+
+/*
+ * 'sighup_handler()' - Handle 'hangup' signals to reconfigure the scheduler.
+ */
+
+static void
+sighup_handler(int sig)			/* I - Signal number */
+{
+  (void)sig;
+
+  NeedReload = RELOAD_ALL;
+  ReloadTime = time(NULL);
+
+#if !defined(HAVE_SIGSET) && !defined(HAVE_SIGACTION)
+  signal(SIGHUP, sighup_handler);
+#endif /* !HAVE_SIGSET && !HAVE_SIGACTION */
+}
+
+
+/*
+ * 'sigterm_handler()' - Handle 'terminate' signals that stop the scheduler.
+ */
+
+static void
+sigterm_handler(int sig)		/* I - Signal number */
+{
+  (void)sig;	/* remove compiler warnings... */
+
+ /*
+  * Flag that we should stop and return...
+  */
+
+  stop_scheduler = 1;
+}
+
+
 /*
  * 'usage()' - Show scheduler usage.
  */
@@ -2030,18 +2051,25 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
 static void
 usage(int status)			/* O - Exit status */
 {
-  _cupsLangPuts(status ? stderr : stdout,
-                _("Usage: cupsd [-c config-file] [-f] [-F] [-h] [-l]\n"
-		  "\n"
-		  "-c config-file      Load alternate configuration file\n"
-		  "-f                  Run in the foreground\n"
-		  "-F                  Run in the foreground but detach\n"
-		  "-h                  Show this usage message\n"
-		  "-l                  Run cupsd from launchd(8)\n"));
+  FILE	*fp = status ? stderr : stdout;	/* Output file */
+
+
+  _cupsLangPuts(fp, _("Usage: cupsd [options]"));
+  _cupsLangPuts(fp, _("Options:"));
+  _cupsLangPuts(fp, _("  -c config-file          Load alternate configuration "
+                      "file."));
+  _cupsLangPuts(fp, _("  -f                      Run in the foreground."));
+  _cupsLangPuts(fp, _("  -F                      Run in the foreground but "
+                      "detach from console."));
+  _cupsLangPuts(fp, _("  -h                      Show this usage message."));
+  _cupsLangPuts(fp, _("  -l                      Run cupsd from launchd(8)."));
+  _cupsLangPuts(fp, _("  -t                      Test the configuration "
+                      "file."));
+
   exit(status);
 }
 
 
 /*
- * End of "$Id: main.c 8657 2009-05-18 15:44:04Z mike $".
+ * End of "$Id: main.c 9783 2011-05-18 20:44:16Z mike $".
  */
