@@ -1,9 +1,9 @@
 /*
- * "$Id: type.c 8601 2009-05-08 05:17:47Z mike $"
+ * "$Id: type.c 9793 2011-05-20 03:49:49Z mike $"
  *
- *   MIME typing routines for the Common UNIX Printing System (CUPS).
+ *   MIME typing routines for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -14,27 +14,23 @@
  *
  * Contents:
  *
- *   mimeAddType()      - Add a MIME type to a database.
- *   mimeAddTypeRule()  - Add a detection rule for a file type.
- *   mimeFileType()     - Determine the type of a file.
- *   mimeType()         - Lookup a file type.
- *   compare_types()    - Compare two MIME super/type names.
- *   checkrules()       - Check each rule in a list.
- *   patmatch()         - Pattern matching...
+ *   mimeAddType()        - Add a MIME type to a database.
+ *   mimeAddTypeRule()    - Add a detection rule for a file type.
+ *   mimeFileType()       - Determine the type of a file.
+ *   mimeType()           - Lookup a file type.
+ *   mime_compare_types() - Compare two MIME super/type names.
+ *   mime_check_rules()   - Check each rule in a list.
+ *   mime_patmatch()      - Pattern matching.
  */
 
 /*
  * Include necessary headers...
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
+#include <cups/string-private.h>
+#include <cups/debug-private.h>
 #include <locale.h>
-
-#include <cups/string.h>
 #include "mime.h"
-#include <cups/debug.h>
 
 
 /*
@@ -54,10 +50,34 @@ typedef struct _mime_filebuf_s		/**** File buffer for MIME typing ****/
  * Local functions...
  */
 
-static int	compare_types(mime_type_t *t0, mime_type_t *t1);
-static int	checkrules(const char *filename, _mime_filebuf_t *fb,
-		           mime_magic_t *rules);
-static int	patmatch(const char *s, const char *pat);
+static int	mime_compare_types(mime_type_t *t0, mime_type_t *t1);
+static int	mime_check_rules(const char *filename, _mime_filebuf_t *fb,
+		                 mime_magic_t *rules);
+static int	mime_patmatch(const char *s, const char *pat);
+
+
+/*
+ * Local globals...
+ */
+
+#ifdef DEBUG
+static const char * const debug_ops[] =
+		{			/* Test names... */
+		  "NOP",		/* No operation */
+		  "AND",		/* Logical AND of all children */
+		  "OR",			/* Logical OR of all children */
+		  "MATCH",		/* Filename match */
+		  "ASCII",		/* ASCII characters in range */
+		  "PRINTABLE",		/* Printable characters (32-255) */
+		  "STRING",		/* String matches */
+		  "CHAR",		/* Character/byte matches */
+		  "SHORT",		/* Short/16-bit word matches */
+		  "INT",		/* Integer/32-bit word matches */
+		  "LOCALE",		/* Current locale matches string */
+		  "CONTAINS",		/* File contains a string */
+		  "ISTRING"		/* Case-insensitive string matches */
+		};
+#endif /* DEBUG */
 
 
 /*
@@ -70,41 +90,60 @@ mimeAddType(mime_t     *mime,		/* I - MIME database */
 	    const char *type)		/* I - Type name */
 {
   mime_type_t	*temp;			/* New MIME type */
+  size_t	typelen;		/* Length of type name */
 
 
+  DEBUG_printf(("mimeAddType(mime=%p, super=\"%s\", type=\"%s\")", mime, super,
+                type));
+
  /*
   * Range check input...
   */
 
   if (!mime || !super || !type)
+  {
+    DEBUG_puts("1mimeAddType: Returning NULL (bad arguments).");
     return (NULL);
+  }
 
  /*
   * See if the type already exists; if so, return the existing type...
   */
 
   if ((temp = mimeType(mime, super, type)) != NULL)
+  {
+    DEBUG_printf(("1mimeAddType: Returning %p (existing).", temp));
     return (temp);
+  }
 
  /*
   * The type doesn't exist; add it...
   */
 
   if (!mime->types)
-    mime->types = cupsArrayNew((cups_array_func_t)compare_types, NULL);
+    mime->types = cupsArrayNew((cups_array_func_t)mime_compare_types, NULL);
 
   if (!mime->types)
+  {
+    DEBUG_puts("1mimeAddType: Returning NULL (no types).");
     return (NULL);
+  }
 
-  if ((temp = calloc(1, sizeof(mime_type_t) - MIME_MAX_TYPE +
-                        strlen(type) + 1)) == NULL)
+  typelen = strlen(type) + 1;
+
+  if ((temp = calloc(1, sizeof(mime_type_t) - MIME_MAX_TYPE + typelen)) == NULL)
+  {
+    DEBUG_puts("1mimeAddType: Returning NULL (out of memory).");
     return (NULL);
+  }
 
   strlcpy(temp->super, super, sizeof(temp->super));
-  strcpy(temp->type, type);		/* Safe: temp->type is allocated */
+  memcpy(temp->type, type, typelen);
+  temp->priority = 100;
 
   cupsArrayAdd(mime->types, temp);
 
+  DEBUG_printf(("1mimeAddType: Returning %p (new).", temp));
   return (temp);
 }
 
@@ -130,6 +169,9 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 		*current;  		/* Current rule */
 
 
+  DEBUG_printf(("mimeAddTypeRule(mt=%p(%s/%s), rule=\"%s\")", mt,
+                mt ? mt->super : "???", mt ? mt->type : "???", rule));
+
  /*
   * Range check input...
   */
@@ -156,8 +198,6 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
   logic  = MIME_MAGIC_NOP;
   invert = 0;
 
-  DEBUG_printf(("%s/%s: %s\n", mt->super, mt->type, rule));
-
   while (*rule != '\0')
   {
     while (isspace(*rule & 255))
@@ -165,13 +205,13 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 
     if (*rule == '(')
     {
-      DEBUG_puts("new parenthesis group");
+      DEBUG_puts("1mimeAddTypeRule: New parenthesis group");
       logic = MIME_MAGIC_NOP;
       rule ++;
     }
     else if (*rule == ')')
     {
-      DEBUG_puts("close paren...");
+      DEBUG_puts("1mimeAddTypeRule: Close paren...");
       if (current == NULL || current->parent == NULL)
         return (-1);
 
@@ -206,11 +246,12 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
         current->prev   = NULL;
 	current->parent = temp;
 
-        DEBUG_printf(("creating new AND group %p...\n", temp));
+        DEBUG_printf(("1mimeAddTypeRule: Creating new AND group %p.", temp));
       }
-      else
+      else if (current->parent)
       {
-        DEBUG_printf(("setting group %p op to AND...\n", current->parent));
+        DEBUG_printf(("1mimeAddTypeRule: Setting group %p op to AND.",
+	              current->parent));
         current->parent->op = MIME_MAGIC_AND;
       }
 
@@ -236,7 +277,8 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	  if ((temp = calloc(1, sizeof(mime_magic_t))) == NULL)
 	    return (-1);
 
-          DEBUG_printf(("creating new AND group %p inside OR group\n", temp));
+          DEBUG_printf(("1mimeAddTypeRule: Creating new AND group %p inside OR "
+	                "group.", temp));
 
           while (current->prev != NULL)
 	  {
@@ -256,7 +298,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	  * This isn't the top rule, so go up one level...
 	  */
 
-          DEBUG_puts("going up one level");
+          DEBUG_puts("1mimeAddTypeRule: Going up one level.");
 	  current = current->parent;
 	}
       }
@@ -266,7 +308,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
     }
     else if (*rule == '!')
     {
-      DEBUG_puts("NOT");
+      DEBUG_puts("1mimeAddTypeRule: NOT");
       invert = 1;
       rule ++;
     }
@@ -360,7 +402,10 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	  length[num_values] = ptr - value[num_values];
 
           if (*rule != ',')
+	  {
+	    num_values ++;
 	    break;
+	  }
 
           rule ++;
 	}
@@ -394,6 +439,11 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	  op = MIME_MAGIC_LOCALE;
 	else if (!strcmp(name, "contains"))
 	  op = MIME_MAGIC_CONTAINS;
+	else if (!strcmp(name, "priority") && num_values == 1)
+	{
+	  mt->priority = atoi(value[0]);
+	  continue;
+	}
 	else
 	  return (-1);
       }
@@ -432,7 +482,8 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
         * Add parenthetical grouping...
 	*/
 
-        DEBUG_printf(("making new OR group %p for parenthesis...\n", temp));
+        DEBUG_printf(("1mimeAddTypeRule: Making new OR group %p for "
+	              "parenthesis.", temp));
 
         temp->op = MIME_MAGIC_OR;
 
@@ -447,8 +498,9 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
         logic = MIME_MAGIC_OR;
       }
 
-      DEBUG_printf(("adding %p: %s, op = %d, logic = %d, invert = %d\n",
-                    temp, name, op, logic, invert));
+      DEBUG_printf(("1mimeAddTypeRule: Adding %p: %s, op=MIME_MAGIC_%s(%d), "
+		    "logic=MIME_MAGIC_%s, invert=%d.", temp, name,
+		    debug_ops[op], op, debug_ops[logic], invert));
 
      /*
       * Fill in data for the rule...
@@ -485,15 +537,18 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	    if (length[1] == 1)
 	      temp->value.charv = value[1][0];
 	    else
-	      temp->value.charv = (char)strtol(value[1], NULL, 0);
+	      temp->value.charv = (unsigned char)strtol(value[1], NULL, 0);
+
+	    DEBUG_printf(("1mimeAddTypeRule: CHAR(%d,0x%02x)", temp->offset,
+	                  temp->value.charv));
 	    break;
 	case MIME_MAGIC_SHORT :
 	    temp->offset       = strtol(value[0], NULL, 0);
-	    temp->value.shortv = (short)strtol(value[1], NULL, 0);
+	    temp->value.shortv = (unsigned short)strtol(value[1], NULL, 0);
 	    break;
 	case MIME_MAGIC_INT :
 	    temp->offset     = strtol(value[0], NULL, 0);
-	    temp->value.intv = (int)strtol(value[1], NULL, 0);
+	    temp->value.intv = (unsigned)strtol(value[1], NULL, 0);
 	    break;
 	case MIME_MAGIC_LOCALE :
 	    if (length[0] > (sizeof(temp->value.localev) - 1))
@@ -531,28 +586,34 @@ mimeFileType(mime_t     *mime,		/* I - MIME database */
 {
   _mime_filebuf_t	fb;		/* File buffer */
   const char		*base;		/* Base filename of file */
-  mime_type_t		*type;		/* File type */
+  mime_type_t		*type,		/* File type */
+			*best;		/* Best match */
 
 
   DEBUG_printf(("mimeFileType(mime=%p, pathname=\"%s\", filename=\"%s\", "
-                "compression=%p)\n",
-                mime, pathname ? pathname : "(nil)",
-		filename ? filename : "(nil)",
-		compression));
+                "compression=%p)", mime, pathname, filename, compression));
 
  /*
   * Range check input parameters...
   */
 
   if (!mime || !pathname)
+  {
+    DEBUG_puts("1mimeFileType: Returning NULL.");
     return (NULL);
+  }
 
  /*
   * Try to open the file...
   */
 
   if ((fb.fp = cupsFileOpen(pathname, "r")) == NULL)
+  {
+    DEBUG_printf(("1mimeFileType: Unable to open \"%s\": %s", pathname,
+                  strerror(errno)));
+    DEBUG_puts("1mimeFileType: Returning NULL.");
     return (NULL);
+  }
 
   fb.offset = -1;
   fb.length = 0;
@@ -577,22 +638,30 @@ mimeFileType(mime_t     *mime,		/* I - MIME database */
   * Then check it against all known types...
   */
 
-  for (type = (mime_type_t *)cupsArrayFirst(mime->types);
+  for (type = (mime_type_t *)cupsArrayFirst(mime->types), best = NULL;
        type;
        type = (mime_type_t *)cupsArrayNext(mime->types))
-    if (checkrules(base, &fb, type->rules))
-      break;
+    if (mime_check_rules(base, &fb, type->rules))
+    {
+      if (!best || type->priority > best->priority)
+        best = type;
+    }
 
  /*
   * Finally, close the file and return a match (if any)...
   */
 
   if (compression)
+  {
     *compression = cupsFileCompression(fb.fp);
+    DEBUG_printf(("1mimeFileType: *compression=%d", *compression));
+  }
 
   cupsFileClose(fb.fp);
 
-  return (type);
+  DEBUG_printf(("1mimeFileType: Returning %p(%s/%s).", best,
+                best ? best->super : "???", best ? best->type : "???"));
+  return (best);
 }
 
 
@@ -605,15 +674,22 @@ mimeType(mime_t     *mime,		/* I - MIME database */
          const char *super,		/* I - Super-type name */
 	 const char *type)		/* I - Type name */
 {
-  mime_type_t	key;			/* MIME type search key*/
+  mime_type_t	key,			/* MIME type search key */
+		*mt;			/* Matching type */
 
 
+  DEBUG_printf(("mimeType(mime=%p, super=\"%s\", type=\"%s\")", mime, super,
+                type));
+
  /*
   * Range check input...
   */
 
   if (!mime || !super || !type)
+  {
+    DEBUG_puts("1mimeType: Returning NULL.");
     return (NULL);
+  }
 
  /*
   * Lookup the type in the array...
@@ -622,36 +698,39 @@ mimeType(mime_t     *mime,		/* I - MIME database */
   strlcpy(key.super, super, sizeof(key.super));
   strlcpy(key.type, type, sizeof(key.type));
 
-  return ((mime_type_t *)cupsArrayFind(mime->types, &key));
+  mt = (mime_type_t *)cupsArrayFind(mime->types, &key);
+  DEBUG_printf(("1mimeType: Returning %p.", mt));
+  return (mt);
 }
 
 
 /*
- * 'compare_types()' - Compare two MIME super/type names.
+ * 'mime_compare_types()' - Compare two MIME super/type names.
  */
 
 static int				/* O - Result of comparison */
-compare_types(mime_type_t *t0,		/* I - First type */
-              mime_type_t *t1)		/* I - Second type */
+mime_compare_types(mime_type_t *t0,	/* I - First type */
+                   mime_type_t *t1)	/* I - Second type */
 {
   int	i;				/* Result of comparison */
 
 
-  if ((i = strcasecmp(t0->super, t1->super)) == 0)
-    i = strcasecmp(t0->type, t1->type);
+  if ((i = _cups_strcasecmp(t0->super, t1->super)) == 0)
+    i = _cups_strcasecmp(t0->type, t1->type);
 
   return (i);
 }
 
 
 /*
- * 'checkrules()' - Check each rule in a list.
+ * 'mime_check_rules()' - Check each rule in a list.
  */
 
 static int				/* O - 1 if match, 0 if no match */
-checkrules(const char      *filename,	/* I - Filename */
-           _mime_filebuf_t *fb,		/* I - File to check */
-           mime_magic_t    *rules)	/* I - Rules to check */
+mime_check_rules(
+    const char      *filename,		/* I - Filename */
+    _mime_filebuf_t *fb,		/* I - File to check */
+    mime_magic_t    *rules)		/* I - Rules to check */
 {
   int		n;			/* Looping var */
   int		region;			/* Region to look at */
@@ -660,27 +739,9 @@ checkrules(const char      *filename,	/* I - Filename */
 		intv;			/* Integer value */
   short		shortv;			/* Short value */
   unsigned char	*bufptr;		/* Pointer into buffer */
-#ifdef DEBUG
-  const char	* const debug_tests[] =	/* Test names... */
-		{
-		  "NOP",		/* No operation */
-		  "AND",		/* Logical AND of all children */
-		  "OR",			/* Logical OR of all children */
-		  "MATCH",		/* Filename match */
-		  "ASCII",		/* ASCII characters in range */
-		  "PRINTABLE",		/* Printable characters (32-255) */
-		  "STRING",		/* String matches */
-		  "CHAR",		/* Character/byte matches */
-		  "SHORT",		/* Short/16-bit word matches */
-		  "INT",		/* Integer/32-bit word matches */
-		  "LOCALE"		/* Current locale matches string */
-		  "CONTAINS"		/* File contains a string */
-		  "ISTRING"		/* Case-insensitive string matches */
-		};
-#endif /* DEBUG */
 
 
-  DEBUG_printf(("checkrules(filename=\"%s\", fb=%p, rules=%p)\n", filename,
+  DEBUG_printf(("4mime_check_rules(filename=\"%s\", fb=%p, rules=%p)", filename,
                 fb, rules));
 
   if (rules == NULL)
@@ -702,7 +763,7 @@ checkrules(const char      *filename,	/* I - Filename */
     switch (rules->op)
     {
       case MIME_MAGIC_MATCH :
-          result = patmatch(filename, rules->value.matchv);
+          result = mime_patmatch(filename, rules->value.matchv);
 	  break;
 
       case MIME_MAGIC_ASCII :
@@ -792,7 +853,7 @@ checkrules(const char      *filename,	/* I - Filename */
 	  break;
 
       case MIME_MAGIC_STRING :
-          DEBUG_printf(("    string(%d, \"%s\")\n", rules->offset,
+          DEBUG_printf(("5mime_check_rules: string(%d, \"%s\")", rules->offset,
 	                rules->value.stringv));
 
          /*
@@ -811,8 +872,8 @@ checkrules(const char      *filename,	/* I - Filename */
 	                              sizeof(fb->buffer));
 	    fb->offset = rules->offset;
 
-            DEBUG_printf(("        loaded %d byte fb->buffer at %d, starts "
-	                  "with \"%c%c%c%c\"...\n",
+            DEBUG_printf(("5mime_check_rules: loaded %d byte fb->buffer at %d, starts "
+	                  "with \"%c%c%c%c\".",
 	                  fb->length, fb->offset, fb->buffer[0], fb->buffer[1],
 			  fb->buffer[2], fb->buffer[3]));
 	  }
@@ -827,7 +888,7 @@ checkrules(const char      *filename,	/* I - Filename */
 	  else
             result = (memcmp(fb->buffer + rules->offset - fb->offset,
 	                     rules->value.stringv, rules->length) == 0);
-          DEBUG_printf(("    result=%d\n", result));
+          DEBUG_printf(("5mime_check_rules: result=%d", result));
 	  break;
 
       case MIME_MAGIC_ISTRING :
@@ -856,7 +917,7 @@ checkrules(const char      *filename,	/* I - Filename */
 	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
 	    result = 0;
 	  else
-            result = (strncasecmp((char *)fb->buffer + rules->offset -
+            result = (_cups_strncasecmp((char *)fb->buffer + rules->offset -
 	                              fb->offset,
 	                          rules->value.stringv, rules->length) == 0);
 	  break;
@@ -1009,7 +1070,7 @@ checkrules(const char      *filename,	/* I - Filename */
 
       default :
           if (rules->child != NULL)
-	    result = checkrules(filename, fb, rules->child);
+	    result = mime_check_rules(filename, fb, rules->child);
 	  else
 	    result = 0;
 	  break;
@@ -1028,8 +1089,8 @@ checkrules(const char      *filename,	/* I - Filename */
     * the the rule set is false...
     */
 
-    DEBUG_printf(("    result of test %p (MIME_MAGIC_%s) is %d\n", rules,
-                  debug_tests[rules->op], result));
+    DEBUG_printf(("5mime_check_rules: result of test %p (MIME_MAGIC_%s) is %d",
+                  rules, debug_ops[rules->op], result));
 
     if ((result && logic == MIME_MAGIC_OR) ||
         (!result && logic == MIME_MAGIC_AND))
@@ -1047,12 +1108,12 @@ checkrules(const char      *filename,	/* I - Filename */
 
 
 /*
- * 'patmatch()' - Pattern matching...
+ * 'mime_patmatch()' - Pattern matching.
  */
 
-static int			/* O - 1 if match, 0 if no match */
-patmatch(const char *s,		/* I - String to match against */
-         const char *pat)	/* I - Pattern to match against */
+static int				/* O - 1 if match, 0 if no match */
+mime_patmatch(const char *s,		/* I - String to match against */
+              const char *pat)		/* I - Pattern to match against */
 {
  /*
   * Range check the input...
@@ -1084,7 +1145,7 @@ patmatch(const char *s,		/* I - String to match against */
 
       while (*s != '\0')
       {
-        if (patmatch(s, pat))
+        if (mime_patmatch(s, pat))
 	  return (1);
 
 	s ++;
@@ -1151,5 +1212,5 @@ patmatch(const char *s,		/* I - String to match against */
 
 
 /*
- * End of "$Id: type.c 8601 2009-05-08 05:17:47Z mike $".
+ * End of "$Id: type.c 9793 2011-05-20 03:49:49Z mike $".
  */
