@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c 8142 2008-11-18 00:43:55Z mike $"
+ * "$Id: lp.c 10495 2012-05-21 22:40:33Z mike $"
  *
- *   "lp" command for the Common UNIX Printing System (CUPS).
+ *   "lp" command for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -17,44 +17,24 @@
  *   main()          - Parse options and send files for printing.
  *   restart_job()   - Restart a job.
  *   set_job_attrs() - Set job attributes.
- *   sighandler()    - Signal catcher for when we print from stdin...
  */
 
 /*
  * Include necessary headers...
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <cups/string.h>
-#include <cups/cups.h>
-#include <cups/i18n.h>
-
-
-#ifndef WIN32
-#  include <unistd.h>
-#  include <signal.h>
+#include <cups/cups-private.h>
 
 
 /*
  * Local functions.
  */
 
-void	sighandler(int);
-#endif /* !WIN32 */
 int	restart_job(const char *command, int job_id);
 int	set_job_attrs(const char *command, int job_id, int num_options,
 	              cups_option_t *options);
 
 
-/*
- * Globals...
- */
-
-char	tempfile[1024];		/* Temporary file for printing from stdin */
-
-
 /*
  * 'main()' - Parse options and send files for printing.
  */
@@ -66,28 +46,19 @@ main(int  argc,				/* I - Number of command-line arguments */
   int		i, j;			/* Looping vars */
   int		job_id;			/* Job ID */
   char		*printer,		/* Printer name */
-		*instance,		/* Instance name */ 
+		*instance,		/* Instance name */
 		*val,			/* Option value */
 		*title;			/* Job title */
   int		priority;		/* Job priority (1-100) */
   int		num_copies;		/* Number of copies per file */
   int		num_files;		/* Number of files to print */
   const char	*files[1000];		/* Files to print */
-  int		num_dests;		/* Number of destinations */
-  cups_dest_t	*dests,			/* Destinations */
-		*dest;			/* Selected destination */
+  cups_dest_t	*dest;			/* Selected destination */
   int		num_options;		/* Number of options */
   cups_option_t	*options;		/* Options */
   int		end_options;		/* No more options? */
   int		silent;			/* Silent or verbose output? */
   char		buffer[8192];		/* Copy buffer */
-  ssize_t	bytes;			/* Bytes copied */
-  off_t		filesize;		/* Size of temp file */
-  int		temp;			/* Temporary file descriptor */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;		/* Signal action */
-  struct sigaction oldaction;		/* Old signal action */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET*/
 
 
 #ifdef __sun
@@ -112,8 +83,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   silent      = 0;
   printer     = NULL;
-  num_dests   = 0;
-  dests       = NULL;
+  dest        = NULL;
   num_options = 0;
   options     = NULL;
   num_files   = 0;
@@ -129,8 +99,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 #ifdef HAVE_SSL
 	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 #else
-            _cupsLangPrintf(stderr,
-	                    _("%s: Sorry, no encryption support compiled in!\n"),
+            _cupsLangPrintf(stderr, _("%s: Sorry, no encryption support."),
 	                    argv[0]);
 #endif /* HAVE_SSL */
 	    break;
@@ -144,16 +113,15 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        _cupsLangPrintf(stderr,
-		                _("%s: Error - expected username after "
-				  "\'-U\' option!\n"),
-		        	argv[0]);
+		                _("%s: Error - expected username after \"-U\" "
+				  "option."), argv[0]);
 	        return (1);
 	      }
 
               cupsSetUser(argv[i]);
 	    }
 	    break;
-	    
+
         case 'c' : /* Copy to spool dir (always enabled) */
 	    break;
 
@@ -168,8 +136,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		        	_("%s: Error - expected destination after "
-			          "\'-d\' option!\n"),
-				argv[0]);
+			          "\"-d\" option."), argv[0]);
 		return (1);
               }
 
@@ -179,13 +146,11 @@ main(int  argc,				/* I - Number of command-line arguments */
             if ((instance = strrchr(printer, '/')) != NULL)
 	      *instance++ = '\0';
 
-	    if (num_dests == 0)
-	      num_dests = cupsGetDests(&dests);
-
-            if ((dest = cupsGetDest(printer, instance, num_dests, dests)) != NULL)
+            if ((dest = cupsGetNamedDest(NULL, printer, instance)) != NULL)
 	    {
 	      for (j = 0; j < dest->num_options; j ++)
-	        if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+	        if (cupsGetOption(dest->options[j].name, num_options,
+		                  options) == NULL)
 	          num_options = cupsAddOption(dest->options[j].name,
 		                              dest->options[j].value,
 					      num_options, &options);
@@ -200,14 +165,14 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        _cupsLangPrintf(stderr,
-		                _("%s: Error - expected form after \'-f\' "
-				  "option!\n"),
+		                _("%s: Error - expected form after \"-f\" "
+				  "option."),
 				argv[0]);
 		return (1);
               }
 	    }
 
-	    _cupsLangPrintf(stderr, _("%s: Warning - form option ignored!\n"),
+	    _cupsLangPrintf(stderr, _("%s: Warning - form option ignored."),
 	                    argv[0]);
 	    break;
 
@@ -222,8 +187,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected hostname after "
-				  "\'-h\' option!\n"),
-				argv[0]);
+				  "\"-h\" option."), argv[0]);
 		return (1);
               }
 
@@ -241,7 +205,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        _cupsLangPrintf(stderr,
-		                _("%s: Expected job ID after \'-i\' option!\n"),
+		                _("%s: Expected job ID after \"-i\" option."),
 				argv[0]);
 		return (1);
               }
@@ -253,8 +217,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    {
 	      _cupsLangPrintf(stderr,
 		              _("%s: Error - cannot print files and alter "
-			        "jobs simultaneously!\n"),
-			      argv[0]);
+			        "jobs simultaneously."), argv[0]);
 	      return (1);
 	    }
 
@@ -265,7 +228,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
             if (job_id < 0)
 	    {
-	      _cupsLangPrintf(stderr, _("%s: Error - bad job ID!\n"), argv[0]);
+	      _cupsLangPrintf(stderr, _("%s: Error - bad job ID."), argv[0]);
 	      break;
 	    }
 	    break;
@@ -299,8 +262,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected copies after "
-				  "\'-n\' option!\n"),
-				argv[0]);
+				  "\"-n\" option."), argv[0]);
 		return (1);
               }
 
@@ -308,12 +270,14 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    }
 
             sprintf(buffer, "%d", num_copies);
-            num_options = cupsAddOption("copies", buffer, num_options, &options);
+            num_options = cupsAddOption("copies", buffer, num_options,
+	                                &options);
 	    break;
 
 	case 'o' : /* Option */
 	    if (argv[i][2] != '\0')
-	      num_options = cupsParseOptions(argv[i] + 2, num_options, &options);
+	      num_options = cupsParseOptions(argv[i] + 2, num_options,
+	                                     &options);
 	    else
 	    {
 	      i ++;
@@ -321,9 +285,8 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        _cupsLangPrintf(stderr,
-		                _("%s: Error - expected option string after "
-				  "\'-o\' option!\n"),
-				argv[0]);
+		                _("%s: Error - expected option=value after "
+				  "\"-o\" option."), argv[0]);
 		return (1);
               }
 
@@ -343,8 +306,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected priority after "
-				  "\'-%c\' option!\n"),
-		        	argv[0], argv[i][1]);
+				  "\"-%c\" option."), argv[0], argv[i][1]);
 		return (1);
               }
 
@@ -366,13 +328,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    {
 	      _cupsLangPrintf(stderr,
 		              _("%s: Error - priority must be between 1 and "
-			        "100.\n"),
-		              argv[0]);
+			        "100."), argv[0]);
 	      return (1);
 	    }
 
             sprintf(buffer, "%d", priority);
-            num_options = cupsAddOption("job-priority", buffer, num_options, &options);
+            num_options = cupsAddOption("job-priority", buffer, num_options,
+	                                &options);
 	    break;
 
 	case 's' : /* Silent */
@@ -390,8 +352,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected title after "
-				  "\'-t\' option!\n"),
-				argv[0]);
+				  "\"-t\" option."), argv[0]);
 		return (1);
               }
 
@@ -408,15 +369,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected mode list after "
-				  "\'-y\' option!\n"),
-				argv[0]);
+				  "\"-y\" option."), argv[0]);
 		return (1);
               }
 	    }
 
 	    _cupsLangPrintf(stderr,
-		            _("%s: Warning - mode option ignored!\n"),
-			    argv[0]);
+		            _("%s: Warning - mode option ignored."), argv[0]);
 	    break;
 
         case 'H' : /* Hold job */
@@ -430,8 +389,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected hold name after "
-				  "\'-H\' option!\n"),
-				argv[0]);
+				  "\"-H\" option."), argv[0]);
 		return (1);
               }
 
@@ -457,9 +415,8 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (job_id < 1)
 	      {
 	        _cupsLangPrintf(stderr,
-		                _("%s: Need job ID (\'-i jobid\') before "
-			          "\'-H restart\'!\n"),
-				argv[0]);
+		                _("%s: Need job ID (\"-i jobid\") before "
+			          "\"-H restart\"."), argv[0]);
 		return (1);
 	      }
 
@@ -482,8 +439,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected page list after "
-				  "\'-P\' option!\n"),
-				argv[0]);
+				  "\"-P\" option."), argv[0]);
 		return (1);
               }
 
@@ -503,14 +459,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected character set after "
-				  "\'-S\' option!\n"),
-				argv[0]);
+				  "\"-S\" option."), argv[0]);
 		return (1);
               }
 	    }
 
 	    _cupsLangPrintf(stderr,
-		            _("%s: Warning - character set option ignored!\n"),
+		            _("%s: Warning - character set option ignored."),
 			    argv[0]);
 	    break;
 
@@ -523,14 +478,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		        	_("%s: Error - expected content type after "
-			          "\'-T\' option!\n"),
-				argv[0]);
+			          "\"-T\" option."), argv[0]);
 		return (1);
               }
 	    }
 
 	    _cupsLangPrintf(stderr,
-		            _("%s: Warning - content type option ignored!\n"),
+		            _("%s: Warning - content type option ignored."),
 			    argv[0]);
 	    break;
 
@@ -539,7 +493,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    break;
 
 	default :
-	    _cupsLangPrintf(stderr, _("%s: Error - unknown option \'%c\'!\n"),
+	    _cupsLangPrintf(stderr, _("%s: Error - unknown option \"%c\"."),
 	                    argv[0], argv[i][1]);
 	    return (1);
       }
@@ -549,8 +503,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       {
         _cupsLangPrintf(stderr,
 			_("%s: Error - cannot print from stdin if files or a "
-		          "job ID are provided!\n"),
-			 argv[0]);
+		          "job ID are provided."), argv[0]);
 	return (1);
       }
 
@@ -564,7 +517,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
       if (access(argv[i], R_OK) != 0)
       {
-        _cupsLangPrintf(stderr, _("%s: Error - unable to access \"%s\" - %s\n"),
+        _cupsLangPrintf(stderr, _("%s: Error - unable to access \"%s\" - %s"),
 	                argv[0], argv[i], strerror(errno));
         return (1);
       }
@@ -581,7 +534,7 @@ main(int  argc,				/* I - Number of command-line arguments */
       }
     }
     else
-      _cupsLangPrintf(stderr, _("%s: Error - too many files - \"%s\"\n"),
+      _cupsLangPrintf(stderr, _("%s: Error - too many files - \"%s\"."),
                       argv[0], argv[i]);
 
  /*
@@ -597,10 +550,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (printer == NULL)
   {
-    if (num_dests == 0)
-      num_dests = cupsGetDests(&dests);
-
-    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    if ((dest = cupsGetNamedDest(NULL, NULL, NULL)) != NULL)
     {
       printer = dest->name;
 
@@ -629,18 +579,17 @@ main(int  argc,				/* I - Number of command-line arguments */
     else
       val = "LPDEST";
 
-    if (printer && !cupsGetDest(printer, NULL, num_dests, dests))
+    if (printer && !cupsGetNamedDest(NULL, printer, NULL))
       _cupsLangPrintf(stderr,
 		      _("%s: Error - %s environment variable names "
-		        "non-existent destination \"%s\"!\n"),
-        	      argv[0], val, printer);
+		        "non-existent destination \"%s\"."), argv[0], val,
+		      printer);
     else if (cupsLastError() == IPP_NOT_FOUND)
       _cupsLangPrintf(stderr,
-		      _("%s: Error - no default destination available.\n"),
+		      _("%s: Error - no default destination available."),
 		      argv[0]);
     else
-      _cupsLangPrintf(stderr,
-		      _("%s: Error - scheduler not responding!\n"),
+      _cupsLangPrintf(stderr, _("%s: Error - scheduler not responding."),
 		      argv[0]);
 
     return (1);
@@ -648,82 +597,51 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (num_files > 0)
     job_id = cupsPrintFiles(printer, num_files, files, title, num_options, options);
-  else
+  else if ((job_id = cupsCreateJob(CUPS_HTTP_DEFAULT, printer,
+                                   title ? title : "(stdin)",
+                                   num_options, options)) > 0)
   {
-    num_files = 1;
+    http_status_t	status;		/* Write status */
+    const char		*format;	/* Document format */
+    ssize_t		bytes;		/* Bytes read */
 
-#ifndef WIN32
-#  if defined(HAVE_SIGSET)
-    sigset(SIGHUP, sighandler);
-    if (sigset(SIGINT, sighandler) == SIG_IGN)
-      sigset(SIGINT, SIG_IGN);
-    sigset(SIGTERM, sighandler);
-#  elif defined(HAVE_SIGACTION)
-    memset(&action, 0, sizeof(action));
-    action.sa_handler = sighandler;
+    if (cupsGetOption("raw", num_options, options))
+      format = CUPS_FORMAT_RAW;
+    else if ((format = cupsGetOption("document-format", num_options,
+                                     options)) == NULL)
+      format = CUPS_FORMAT_AUTO;
 
-    sigaction(SIGHUP, &action, NULL);
-    sigaction(SIGINT, NULL, &oldaction);
-    if (oldaction.sa_handler != SIG_IGN)
-      sigaction(SIGINT, &action, NULL);
-    sigaction(SIGTERM, &action, NULL);
-#  else
-    signal(SIGHUP, sighandler);
-    if (signal(SIGINT, sighandler) == SIG_IGN)
-      signal(SIGINT, SIG_IGN);
-    signal(SIGTERM, sighandler);
-#  endif
-#endif /* !WIN32 */
+    status = cupsStartDocument(CUPS_HTTP_DEFAULT, printer, job_id, NULL,
+                               format, 1);
 
-    temp = cupsTempFd(tempfile, sizeof(tempfile));
+    while (status == HTTP_CONTINUE &&
+           (bytes = read(0, buffer, sizeof(buffer))) > 0)
+      status = cupsWriteRequestData(CUPS_HTTP_DEFAULT, buffer, bytes);
 
-    if (temp < 0)
+    if (status != HTTP_CONTINUE)
     {
-      _cupsLangPrintf(stderr,
-		      _("%s: Error - unable to create temporary file \"%s\" - %s\n"),
-        	      argv[0], tempfile, strerror(errno));
+      _cupsLangPrintf(stderr, _("%s: Error - unable to queue from stdin - %s."),
+		      argv[0], httpStatus(status));
+      cupsFinishDocument(CUPS_HTTP_DEFAULT, printer);
+      cupsCancelJob2(CUPS_HTTP_DEFAULT, printer, job_id, 0);
       return (1);
     }
 
-    while ((bytes = read(0, buffer, sizeof(buffer))) > 0)
-      if (write(temp, buffer, bytes) < 0)
-      {
-	_cupsLangPrintf(stderr,
-		        _("%s: Error - unable to write to temporary file "
-			  "\"%s\" - %s\n"),
-        	        argv[0], tempfile, strerror(errno));
-        close(temp);
-        unlink(tempfile);
-	return (1);
-      }
-
-    filesize = lseek(temp, 0, SEEK_CUR);
-    close(temp);
-
-    if (filesize <= 0)
+    if (cupsFinishDocument(CUPS_HTTP_DEFAULT, printer) != IPP_OK)
     {
-      _cupsLangPrintf(stderr,
-		      _("%s: Error - stdin is empty, so no job has been sent.\n"),
-		      argv[0]);
-      unlink(tempfile);
+      _cupsLangPrintf(stderr, "%s: %s", argv[0], cupsLastErrorString());
+      cupsCancelJob2(CUPS_HTTP_DEFAULT, printer, job_id, 0);
       return (1);
     }
-
-    if (title)
-      job_id = cupsPrintFile(printer, tempfile, title, num_options, options);
-    else
-      job_id = cupsPrintFile(printer, tempfile, "(stdin)", num_options, options);
-
-    unlink(tempfile);
   }
 
   if (job_id < 1)
   {
-    _cupsLangPrintf(stderr, "%s: %s\n", argv[0], cupsLastErrorString());
+    _cupsLangPrintf(stderr, "%s: %s", argv[0], cupsLastErrorString());
     return (1);
   }
   else if (!silent)
-    _cupsLangPrintf(stdout, _("request id is %s-%d (%d file(s))\n"),
+    _cupsLangPrintf(stdout, _("request id is %s-%d (%d file(s))"),
 		    printer, job_id, num_files);
 
   return (0);
@@ -738,13 +656,10 @@ int					/* O - Exit status */
 restart_job(const char *command,	/* I - Command name */
             int        job_id)		/* I - Job ID */
 {
-  http_t	*http;			/* HTTP connection to server */
   ipp_t		*request;		/* IPP request */
   char		uri[HTTP_MAX_URI];	/* URI for job */
 
 
-  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
-
   request = ippNewRequest(IPP_RESTART_JOB);
 
   sprintf(uri, "ipp://localhost/jobs/%d", job_id);
@@ -755,11 +670,11 @@ restart_job(const char *command,	/* I - Command name */
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
                "requesting-user-name", NULL, cupsUser());
 
-  ippDelete(cupsDoRequest(http, request, "/jobs"));
+  ippDelete(cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/jobs"));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
   {
-    _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+    _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
     return (1);
   }
 
@@ -777,7 +692,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
               int           num_options,/* I - Number of options */
 	      cups_option_t *options)	/* I - Options */
 {
-  http_t	*http;			/* HTTP connection to server */
   ipp_t		*request;		/* IPP request */
   char		uri[HTTP_MAX_URI];	/* URI for job */
 
@@ -785,8 +699,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
   if (num_options == 0)
     return (0);
 
-  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
-
   request = ippNewRequest(IPP_SET_JOB_ATTRIBUTES);
 
   sprintf(uri, "ipp://localhost/jobs/%d", job_id);
@@ -799,11 +711,11 @@ set_job_attrs(const char    *command,	/* I - Command name */
 
   cupsEncodeOptions(request, num_options, options);
 
-  ippDelete(cupsDoRequest(http, request, "/jobs"));
+  ippDelete(cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/jobs"));
 
   if (cupsLastError() > IPP_OK_CONFLICT)
   {
-    _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+    _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
     return (1);
   }
 
@@ -811,29 +723,6 @@ set_job_attrs(const char    *command,	/* I - Command name */
 }
 
 
-#ifndef WIN32
 /*
- * 'sighandler()' - Signal catcher for when we print from stdin...
- */
-
-void
-sighandler(int s)			/* I - Signal number */
-{
- /*
-  * Remove the temporary file we're using to print from stdin...
-  */
-
-  unlink(tempfile);
-
- /*
-  * Exit...
-  */
-
-  exit(s);
-}
-#endif /* !WIN32 */
-
-
-/*
- * End of "$Id: lp.c 8142 2008-11-18 00:43:55Z mike $".
+ * End of "$Id: lp.c 10495 2012-05-21 22:40:33Z mike $".
  */
